import logging
import os
import re
import time
from logging import Logger
from typing import Any, Callable, Dict, List, Optional, Type, Union
from urllib.parse import SplitResult, urlsplit

from bs4 import BeautifulSoup
from pydantic import AnyHttpUrl, TypeAdapter
from requests import PreparedRequest, Request, Response

from . import utils
from .__about__ import __version__
from .adapter import HTTPAdapter
from .exception import (
    ParseModelError,
    ServerHtmlError,
    ServerJsonError,
    ServerMultiJsonError,
    ServerTextError,
)
from .model.common import Error, MsError, MsStatus
from .model.enum import Api, HttpMethod, Status, TaskAction
from .model.request import EndpointRequest
from .model.response import (
    MR,
    BaseLinkableResp,
    BaseTaskResp,
    BytesResp,
    C,
    ConsumeLinkableResp,
    GetAlertNoteResp,
    GetAlertResp,
    GetApiKeyResp,
    GetEndpointDetailsResp,
    GetOatPackageResp,
    GetPipelineResp,
    MultiApiKeyResp,
    MultiResp,
    MultiUrlResp,
    NoContentResp,
    R,
    S,
    SandboxSubmissionStatusResp,
    T,
    TextResp,
)
from .result import multi_result, result

USERAGENT_SUFFIX: str = "PyTMV1"
API_VERSION: str = "v3.0"

log: Logger = logging.getLogger(__name__)


class Core:
    def __init__(
        self,
        appname: str,
        token: str,
        url: str,
        pool_connections: int,
        pool_maxsize: int,
        connect_timeout: int,
        read_timeout: int,
    ):
        self._adapter = HTTPAdapter(pool_connections, pool_maxsize, 0, True)
        self._c_timeout = connect_timeout
        self._r_timeout = read_timeout
        self._appname = appname
        self._token = token
        self._url = str(TypeAdapter(AnyHttpUrl).validate_python(_format(url)))
        self._headers: Dict[str, str] = {
            "Authorization": f"Bearer {self._token}",
            "User-Agent": f"{self._appname}-{USERAGENT_SUFFIX}/{__version__}",
        }
        self._proxies: Optional[Dict[str, str]] = _proxy(
            os.getenv("HTTP_PROXY") or os.getenv("http_proxy"),
            os.getenv("HTTPS_PROXY") or os.getenv("https_proxy"),
        )

    @result
    def send(
        self,
        class_: Type[R],
        api: str,
        method: HttpMethod = HttpMethod.GET,
        **kwargs: Any,
    ) -> R:
        return self._process(
            class_,
            api,
            method,
            **kwargs,
        )

    @multi_result
    def send_endpoint(
        self,
        api: Api,
        *tasks: EndpointRequest,
    ) -> MultiResp:
        return self._process(
            MultiResp,
            api,
            HttpMethod.POST,
            json=[
                task.model_dump(by_alias=True, exclude_none=True)
                for task in tasks
            ],
        )

    @result
    def send_linkable(
        self,
        class_: Type[BaseLinkableResp[C]],
        api: str,
        consumer: Callable[[C], None],
        **kwargs: Any,
    ) -> ConsumeLinkableResp:
        return ConsumeLinkableResp(
            total_consumed=self._consume_linkable(
                lambda: self._process(
                    class_,
                    api,
                    **kwargs,
                ),
                consumer,
                kwargs.get("headers", {}),
            )
        )

    @multi_result
    def send_multi(
        self,
        class_: Type[MR],
        api: str,
        **kwargs: Any,
    ) -> MR:
        return self._process(
            class_,
            api,
            HttpMethod.POST,
            **kwargs,
        )

    @result
    def send_sandbox_result(
        self,
        class_: Type[R],
        api: Api,
        submit_id: str,
        poll: bool,
        poll_time_sec: float,
    ) -> R:
        if poll:
            _poll_status(
                lambda: self._process(
                    SandboxSubmissionStatusResp,
                    Api.GET_SANDBOX_SUBMISSION_STATUS.value.format(submit_id),
                ),
                poll_time_sec,
            )
        return self._process(class_, api.value.format(submit_id))

    @result
    def send_task_result(
        self, class_: Type[T], task_id: str, poll: bool, poll_time_sec: float
    ) -> T:
        status_call: Callable[[], T] = lambda: self._process(
            class_,
            Api.GET_TASK_RESULT.value.format(task_id),
        )
        if poll:
            return _poll_status(
                status_call,
                poll_time_sec,
            )
        return status_call()

    def _consume_linkable(
        self,
        api_call: Callable[[], BaseLinkableResp[C]],
        consumer: Callable[[C], None],
        headers: Dict[str, str],
        count: int = 0,
    ) -> int:
        total_count: int = count
        response: BaseLinkableResp[C] = api_call()
        for item in response.items:
            consumer(item)
            total_count += 1
        if response.next_link:
            sr: SplitResult = urlsplit(response.next_link)
            log.debug("Found nextLink")
            return self._consume_linkable(
                lambda: self._process(
                    type(response),
                    f"{sr.path[5:]}?{sr.query}",
                    headers=headers,
                ),
                consumer,
                headers,
                total_count,
            )
        log.debug(
            "Records consumed: [Total=%s, Type=%s]",
            total_count,
            type(
                response.items[0] if len(response.items) > 0 else response
            ).__name__,
        )
        return total_count

    def _process(
        self,
        class_: Type[R],
        uri: str,
        method: HttpMethod = HttpMethod.GET,
        **kwargs: Any,
    ) -> R:
        log.debug(
            "Processing request [Method=%s, Class=%s, URI=%s, Options=%s]",
            method.value,
            class_.__name__,
            uri,
            kwargs,
        )
        raw_response: Response = self._send_internal(
            self._prepare(uri, method, **kwargs)
        )
        _validate(raw_response)
        return _parse_data(raw_response, class_)

    def _prepare(
        self, uri: str, method: HttpMethod, **kwargs: Any
    ) -> PreparedRequest:
        return Request(
            method.value,
            self._url + uri,
            headers={**self._headers, **kwargs.pop("headers", {})},
            **kwargs,
        ).prepare()

    def _send_internal(self, request: PreparedRequest) -> Response:
        log.info(
            "Sending request [Method=%s, URL=%s, Headers=%s, Body=%s]",
            request.method,
            request.url,
            re.sub("Bearer \\S+", "*****", str(request.headers)),
            _hide_binary(request),
        )
        response: Response = self._adapter.send(
            request,
            timeout=(self._c_timeout, self._r_timeout),
            proxies=self._proxies,
        )
        log.info(
            "Received response [Status=%s, Headers=%s, Body=%s]",
            response.status_code,
            response.headers,
            _hide_binary(response),
        )
        return response


def _proxy(
    http: Optional[str], https: Optional[str]
) -> Optional[Dict[str, str]]:
    proxies = {}
    if http:
        proxies["http"] = http
    if https:
        proxies["https"] = https
    log.debug("Proxy settings: %s", proxies)
    return proxies if len(proxies.items()) > 0 else None


def _format(url: str) -> str:
    return (url if url.endswith("/") else url + "/") + API_VERSION


def _hide_binary(http_object: Union[PreparedRequest, Response]) -> str:
    content_type = http_object.headers.get("Content-Type", "")
    if "json" not in content_type and "application" in content_type:
        return "***binary content***"
    if "multipart/form-data" in content_type:
        return "***file content***"
    if isinstance(http_object, Response):
        return http_object.text
    return str(http_object.body)


def _is_http_success(status_codes: List[int]) -> bool:
    return len(list(filter(lambda s: not 200 <= s < 399, status_codes))) == 0


def _parse_data(raw_response: Response, class_: Type[R]) -> R:
    content_type = raw_response.headers.get("Content-Type", "")
    if raw_response.status_code == 201:
        return class_(**raw_response.headers)
    if raw_response.status_code == 204 and class_ == NoContentResp:
        return class_()
    if "text" in content_type and class_ == TextResp:
        return class_.model_construct(text=raw_response.text)
    if "json" in content_type or "text" in content_type:
        log.debug("Parsing json response [Class=%s]", class_.__name__)
        if class_ in [MultiResp, MultiUrlResp, MultiApiKeyResp]:
            return class_(items=raw_response.json())
        if class_ == GetEndpointDetailsResp:
            return class_(data=raw_response.json())
        if class_ == GetOatPackageResp:
            return class_(package=raw_response.json())
        if class_ in [
            GetAlertResp,
            GetApiKeyResp,
            GetAlertNoteResp,
            GetPipelineResp,
        ]:
            return class_(
                data=raw_response.json(),
                etag=raw_response.headers.get("ETag", ""),
            )
        if class_ == BaseTaskResp:
            resp_class: Type[BaseTaskResp] = utils.task_action_resp_class(
                TaskAction(raw_response.json()["action"])
            )
            class_ = resp_class if issubclass(resp_class, class_) else class_
        return class_(**raw_response.json())
    if (
        "application" in content_type
        or "gzip" == raw_response.headers.get("Content-Encoding")
    ) and class_ == BytesResp:
        log.debug("Parsing binary response")
        return class_.model_construct(content=raw_response.content)
    raise ParseModelError(class_.__name__, raw_response)


def _parse_html(html: str) -> str:
    log.info("Parsing html response [Html=%s]", html)
    soup = BeautifulSoup(html, "html.parser")
    return "\n".join(
        line.strip() for line in soup.text.split("\n") if line.strip()
    )


def _poll_status(
    status_call: Callable[[], S],
    poll_time_sec: float,
) -> S:
    start_time: float = time.time()
    elapsed_time: float = 0
    response: S = status_call()
    while elapsed_time < poll_time_sec:
        if response.status in [Status.QUEUED, Status.RUNNING]:
            time.sleep(2)
            response = status_call()
            elapsed_time = time.time() - start_time
        else:
            break
    return response


def _validate(raw_response: Response) -> None:
    log.debug("Validating response [%s]", raw_response)
    content_type: str = raw_response.headers.get("Content-Type", "")
    if not _is_http_success([raw_response.status_code]):
        if "application/json" in content_type:
            error: Dict[str, Any] = raw_response.json().get("error")
            error["status"] = raw_response.status_code
            raise ServerJsonError(
                Error(**error),
            )
        if "text/html" in content_type:
            raise ServerHtmlError(
                raw_response.status_code, _parse_html(raw_response.text)
            )
        raise ServerTextError(raw_response.status_code, raw_response.text)
    if raw_response.status_code == 207:
        if not _is_http_success(
            MsStatus(
                root=[int(d.get("status", 500)) for d in raw_response.json()]
            ).values()
        ):
            raise ServerMultiJsonError(
                [
                    MsError.model_validate(error)
                    for error in raw_response.json()
                ]
            )



from typing import Callable, Optional

from .. import utils
from ..core import Core
from ..model.common import AlertNote
from ..model.enum import Api, HttpMethod, QueryOp
from ..model.response import (
    AddAlertNoteResp,
    ConsumeLinkableResp,
    GetAlertNoteResp,
    ListAlertNoteResp,
    NoContentResp,
)
from ..result import Result


class Note:
    _core: Core

    def __init__(self, core: Core):
        self._core = core

    def create(
        self, alert_id: str, note_content: str
    ) -> Result[AddAlertNoteResp]:
        """Adds a note to the specified Workbench alert.

        :param alert_id: Workbench alert id.
        :type alert_id: str
        :param note_content: Value of the note.
        :type note_content: str
        :rtype: Result[AddAlertNoteResp]:
        """
        return self._core.send(
            AddAlertNoteResp,
            Api.ADD_ALERT_NOTE.value.format(alert_id),
            HttpMethod.POST,
            json={"content": note_content},
        )

    def update(
        self, alert_id: str, note_id: str, etag: str, note_content: str
    ) -> Result[NoContentResp]:
        """Updates the content of the specified Workbench alert note.

        :param alert_id: Workbench alert id.
        :type alert_id: str
        :param note_id: Workbench alert note id.
        :type note_id: str
        :param etag: Workbench alert note ETag.
        :param note_content: Content of the alert note.
        :return: Result[NoContentResp]
        """
        return self._core.send(
            NoContentResp,
            Api.UPDATE_ALERT_NOTE.value.format(alert_id, note_id),
            HttpMethod.PATCH,
            json={"content": note_content},
            headers={
                "If-Match": etag if etag.startswith('"') else '"' + etag + '"'
            },
        )

    def delete(
        self,
        alert_id: str,
        *note_ids: str,
    ) -> Result[NoContentResp]:
        """Deletes the specified notes from a Workbench alert.

        :param alert_id: Workbench alert id.
        :type alert_id: str
        :param note_ids: Workbench alert note ids.
        :type note_ids: Tuple[str, ...]
        :return: Result[NoContentResp]
        """
        return self._core.send(
            NoContentResp,
            Api.DELETE_ALERT_NOTE.value.format(alert_id),
            HttpMethod.POST,
            json=[{"id": note_id} for note_id in note_ids],
        )

    def get(self, alert_id: str, note_id: str) -> Result[GetAlertNoteResp]:
        """Retrieves the specified Workbench alert note.

        :param alert_id: Workbench alert id.
        :type alert_id: str
        :param note_id: Workbench alert note id.
        :type note_id: str
        :return:
        """
        return self._core.send(
            GetAlertNoteResp,
            Api.GET_ALERT_NOTE.value.format(alert_id, note_id),
        )

    def list(
        self,
        alert_id: str,
        top: int = 50,
        start_time: Optional[str] = None,
        end_time: Optional[str] = None,
        op: QueryOp = QueryOp.AND,
        **fields: str,
    ) -> Result[ListAlertNoteResp]:
        """Retrieves workbench alert notes in a paginated list.

        :param alert_id: Workbench alert id.
        :type alert_id: str
        :param top: Number of records fetched per page.
        :type top: int
        :param start_time: Date that indicates the start of the data retrieval
        time range (yyyy-MM-ddThh:mm:ssZ).
        :type start_time: Optional[str]
        :param end_time: Date that indicates the end of the data retrieval
        time range (yyyy-MM-ddThh:mm:ssZ).
        :type end_time: Optional[str]
        :param op: Operator to apply between fields (ie: ... OR ...).
        :type op: QueryOp
        :param fields: Field/value used to filter result (i.e:id="1"),
        check Vision One API documentation for full list of supported fields.
        :type fields: Dict[str, str]
        :rtype: Result[GetAlertNoteListResp]:
        """
        return self._core.send(
            ListAlertNoteResp,
            Api.GET_ALERT_NOTE_LIST.value.format(alert_id),
            params=utils.filter_none(
                {
                    "startDateTime": start_time,
                    "endDateTime": end_time,
                    "orderBy": "createdDateTime desc",
                    "top": top,
                }
            ),
            headers=utils.tmv1_filter(op, fields),
        )

    def consume(
        self,
        consumer: Callable[[AlertNote], None],
        alert_id: str,
        top: int = 50,
        start_time: Optional[str] = None,
        end_time: Optional[str] = None,
        op: QueryOp = QueryOp.AND,
        **fields: str,
    ) -> Result[ConsumeLinkableResp]:
        """Retrieves workbench alert notes in a paginated list.

        :param consumer: Function which will consume every record in result.
        :type consumer: Callable[[AlertNote], None]
        :param alert_id: Workbench alert id.
        :type alert_id: str
        :param top: Number of records fetched per page.
        :type top: int
        :param start_time: Date that indicates the start of the data retrieval
        time range (yyyy-MM-ddThh:mm:ssZ).
        :type start_time: Optional[str]
        :param end_time: Date that indicates the end of the data retrieval
        time range (yyyy-MM-ddThh:mm:ssZ).
        :type end_time: Optional[str]
        :param op: Operator to apply between fields (ie: ... OR ...).
        :type op: QueryOp
        :param fields: Field/value used to filter result (i.e:id="1"),
        check Vision One API documentation for full list of supported fields.
        :type fields: Dict[str, str]
        :rtype: Result[ConsumeLinkableResp]:
        """
        return self._core.send_linkable(
            ListAlertNoteResp,
            Api.GET_ALERT_NOTE_LIST.value.format(alert_id),
            consumer,
            params=utils.filter_none(
                {
                    "startDateTime": start_time,
                    "endDateTime": end_time,
                    "orderBy": "createdDateTime desc",
                    "top": top,
                }
            ),
            headers=utils.tmv1_filter(op, fields),
        )



from typing import Callable, List, Optional, Union

from .. import utils
from ..core import Core
from ..model.common import EmailActivity
from ..model.enum import Api, QueryOp, SearchMode
from ..model.request import EmailMessageIdRequest, EmailMessageUIdRequest
from ..model.response import (
    ConsumeLinkableResp,
    GetEmailActivitiesCountResp,
    ListEmailActivityResp,
    MultiResp,
)
from ..result import MultiResult, Result


class Email:
    _core: Core

    def __init__(self, core: Core):
        self._core = core

    def quarantine(
        self, *messages: Union[EmailMessageUIdRequest, EmailMessageIdRequest]
    ) -> MultiResult[MultiResp]:
        """Quarantine a message from one or more mailboxes.

        :param messages: Message(s) to quarantine.
        :type messages: Tuple[EmailUIdTask, EmailMsgIdTask, ...]
        :rtype: MultiResult[MultiResp]
        """
        return self._core.send_multi(
            MultiResp,
            Api.QUARANTINE_EMAIL_MESSAGE,
            json=[
                task.model_dump(by_alias=True, exclude_none=True)
                for task in messages
            ],
        )

    def restore(
        self, *messages: Union[EmailMessageUIdRequest, EmailMessageIdRequest]
    ) -> MultiResult[MultiResp]:
        """Restore quarantined email message(s).

        :param messages: Message(s) to restore.
        :type messages: Tuple[EmailUIdTask, EmailMsgIdTask, ...]
        :rtype: MultiResult[MultiResp]
        """
        return self._core.send_multi(
            MultiResp,
            Api.RESTORE_EMAIL_MESSAGE,
            json=[
                task.model_dump(by_alias=True, exclude_none=True)
                for task in messages
            ],
        )

    def delete(
        self, *messages: Union[EmailMessageUIdRequest, EmailMessageIdRequest]
    ) -> MultiResult[MultiResp]:
        """Deletes a message from one or more mailboxes.

        :param messages: Message(s) to delete.
        :type messages: Tuple[EmailUIdTask, EmailMsgIdTask, ...]
        :rtype: MultiResult[MultiResp]
        """
        return self._core.send_multi(
            MultiResp,
            Api.DELETE_EMAIL_MESSAGE,
            json=[
                task.model_dump(by_alias=True, exclude_none=True)
                for task in messages
            ],
        )

    def get_activity_count(
        self,
        start_time: Optional[str] = None,
        end_time: Optional[str] = None,
        select: Optional[List[str]] = None,
        top: int = 500,
        op: QueryOp = QueryOp.AND,
        **fields: str,
    ) -> Result[GetEmailActivitiesCountResp]:
        """Retrieves the count of email activity data in a paginated list
         filtered by provided values.

        :param start_time: Date that indicates the start of the data retrieval
        time range (yyyy-MM-ddThh:mm:ssZ).
        Defaults to 24 hours before the request is made.
        :type start_time: Optional[str]
        :param end_time: Date that indicates the end of the data retrieval
        time range (yyyy-MM-ddThh:mm:ssZ).
        Defaults to the time the request is made.
        :type end_time: Optional[str]
        :param select: List of fields to include in the search results,
        if no fields are specified, the query returns all supported fields.
        :type select: Optional[List[str]]
        :param top: Number of records fetched per page.
        :type top: int
        :param op: Operator to apply between fields (ie: uuid=... OR tags=...)
        :type op: QueryOp
        :param fields: Field/value used to filter result (ie: uuid="123456")
        check Vision One API documentation for full list of supported fields.
        :type fields: Dict[str, str]
        :rtype: Result[GetEmailActivityDataCountResp]:
        """
        return self._core.send(
            GetEmailActivitiesCountResp,
            Api.GET_EMAIL_ACTIVITY_DATA,
            params=utils.build_activity_request(
                start_time,
                end_time,
                select,
                top,
                SearchMode.COUNT_ONLY,
            ),
            headers=utils.tmv1_activity_query(op, fields),
        )

    def list_activity(
        self,
        start_time: Optional[str] = None,
        end_time: Optional[str] = None,
        select: Optional[List[str]] = None,
        top: int = 500,
        op: QueryOp = QueryOp.AND,
        **fields: str,
    ) -> Result[ListEmailActivityResp]:
        """Retrieves email activity data in a paginated list
         filtered by provided values.

        :param start_time: Date that indicates the start of the data retrieval
        time range (yyyy-MM-ddThh:mm:ssZ).
        Defaults to 24 hours before the request is made.
        :type start_time: Optional[str]
        :param end_time: Date that indicates the end of the data retrieval
        time range (yyyy-MM-ddThh:mm:ssZ).
        Defaults to the time the request is made.
        :type end_time: Optional[str]
        :param select: List of fields to include in the search results,
        if no fields are specified, the query returns all supported fields.
        :type select: Optional[List[str]]
        :param top: Number of records fetched per page.
        :type top: int
        :param op: Operator to apply between fields (ie: uuid=... OR tags=...)
        :type op: QueryOp
        :param fields: Field/value used to filter result (ie: uuid="123456")
        check Vision One API documentation for full list of supported fields.
        :type fields: Dict[str, str]
        :rtype: Result[GetEmailActivityDataResp]:
        """
        return self._core.send(
            ListEmailActivityResp,
            Api.GET_EMAIL_ACTIVITY_DATA,
            params=utils.build_activity_request(
                start_time,
                end_time,
                select,
                top,
                SearchMode.DEFAULT,
            ),
            headers=utils.tmv1_activity_query(op, fields),
        )

    def consume_activity(
        self,
        consumer: Callable[[EmailActivity], None],
        start_time: Optional[str] = None,
        end_time: Optional[str] = None,
        select: Optional[List[str]] = None,
        top: int = 500,
        op: QueryOp = QueryOp.AND,
        **fields: str,
    ) -> Result[ConsumeLinkableResp]:
        """Retrieves and consume email activity data in a paginated list
         filtered by provided values.

        :param consumer: Function which will consume every record in result.
        :type consumer: Callable[[EmailActivity], None]
        :param start_time: Date that indicates the start of the data retrieval
        time range (yyyy-MM-ddThh:mm:ssZ).
        Defaults to 24 hours before the request is made.
        :type start_time: Optional[str]
        :param end_time: Date that indicates the end of the data retrieval
        time range (yyyy-MM-ddThh:mm:ssZ).
        Defaults to the time the request is made.
        :type end_time: Optional[str]
        :param select: List of fields to include in the search results,
        if no fields are specified, the query returns all supported fields.
        :type select: Optional[List[str]]
        :param top: Number of records fetched per page.
        :type top: int
        :param op: Operator to apply between fields (ie: uuid=... OR tags=...)
        :type op: QueryOp
        :param fields: Field/value used to filter result (ie: uuid="123456")
        check Vision One API documentation for full list of supported fields.
        :type fields: Dict[str, str]
        :rtype: Result[ConsumeLinkableResp]:
        """
        return self._core.send_linkable(
            ListEmailActivityResp,
            Api.GET_EMAIL_ACTIVITY_DATA,
            consumer,
            params=utils.build_activity_request(
                start_time,
                end_time,
                select,
                top,
                SearchMode.DEFAULT,
            ),
            headers=utils.tmv1_activity_query(op, fields),
        )



from __future__ import annotations

from typing import Any, Dict, List, Optional, Tuple, Union

from pydantic import BaseModel as PydanticBaseModel
from pydantic import ConfigDict, Field
from pydantic import RootModel as PydanticRootModel
from pydantic import field_validator, model_validator
from pydantic.alias_generators import to_camel

from .enum import (
    AlertStatus,
    ApiStatus,
    DetectionType,
    EntityType,
    Iam,
    IntegrityLevel,
    InvestigationResult,
    InvestigationStatus,
    OatDataSource,
    OatEntityType,
    OatRiskLevel,
    ObjectType,
    OperatingSystem,
    ProductCode,
    Provider,
    RiskLevel,
    ScanAction,
    ScriptType,
    Severity,
    Status,
)

CFG = ConfigDict(alias_generator=to_camel, populate_by_name=True)
CFG_NAMESPACE = ConfigDict(**CFG, protected_namespaces=())


class BaseModel(PydanticBaseModel):
    model_config = CFG


class RootModel(PydanticRootModel[List[int]]):
    model_config = CFG


class BaseConsumable(BaseModel): ...


class Account(BaseModel):
    account_name: str
    iam: Iam
    last_action_date_time: str
    status: Status


class Alert(BaseConsumable):
    model_config = CFG_NAMESPACE

    id: str
    schema_version: str
    status: AlertStatus
    # Deprecated
    investigation_status: InvestigationStatus
    investigation_result: InvestigationResult
    workbench_link: str
    alert_provider: Provider
    model: str
    model_type: DetectionType
    score: int
    severity: Severity
    impact_scope: ImpactScope
    created_date_time: str
    updated_date_time: str
    first_investigated_date_time: Optional[str] = None
    incident_id: Optional[str] = None
    case_id: Optional[str] = None
    owner_ids: List[str] = Field(default=[])
    model_id: str


class AlertNote(BaseConsumable):
    id: int
    content: str
    creator_name: str
    created_date_time: str
    creator_mail_address: Optional[str]
    last_updated_by: Optional[str]
    last_updated_date_time: Optional[str]


class ApiKey(BaseConsumable):
    id: str
    name: str
    status: ApiStatus
    role: str
    expired_date_time: str
    last_used_date_time: str
    description: Optional[str]


class Digest(BaseModel):
    md5: str
    sha1: str
    sha256: str


class EmailMessage(BaseModel):
    last_action_date_time: str
    message_id: Optional[str] = None
    mail_box: Optional[str] = None
    message_subject: Optional[str] = None
    unique_id: Optional[str] = None
    organization_id: Optional[str] = None
    status: Status


class Value(BaseModel):
    updated_date_time: str
    value: str


class ValueList(BaseModel):
    updated_date_time: str
    value: List[str]


class Endpoint(BaseConsumable):
    agent_guid: str
    login_account: ValueList
    endpoint_name: Value
    mac_address: ValueList
    ip: ValueList
    os_name: Optional[OperatingSystem] = None
    os_version: Optional[str] = None
    os_description: Optional[str] = None
    product_code: Optional[ProductCode] = None
    installed_product_codes: List[ProductCode] = Field(default=[])
    component_update_policy: Optional[str] = None
    component_update_status: Optional[str] = None
    component_version: Optional[str] = None
    policy_name: Optional[str] = None
    protection_manager: Optional[str] = None

    @field_validator("os_name", "product_code", mode="before")
    @classmethod
    def allow_empty_enum_string(cls, value: Any) -> Union[Any, None]:
        if value == "":
            return None
        return value


class EppAgentVirtualMachineDetails(BaseModel):
    cloud_instance_id: Optional[str] = None
    cloud_provider: Optional[str] = None
    cloud_account_id: Optional[str] = None
    power_state: Optional[str] = None


class EppAgentPattern(BaseModel):
    id: str
    name: str


class EppAgent(BaseModel):
    endpoint_group: Optional[str] = None
    protection_manager: Optional[str] = None
    product_names: List[str] = Field(default=[])
    policy_name: Optional[str] = None
    status: Optional[str] = None
    last_connected_date_time: Optional[str] = None
    version: Optional[str] = None
    last_scanned_date_time: Optional[str] = None
    component_version: Optional[str] = None
    component_update_policy: Optional[str] = None
    component_update_status: Optional[str] = None
    kernel_support_package_version: Optional[str] = None
    installed_component_ids: List[str] = Field(default=[])
    patterns: List[EppAgentPattern] = Field(default=[])
    virtual_machine_details: Optional[EppAgentVirtualMachineDetails] = None


class EdrSensorPattern(BaseModel):
    id: str
    name: str


class EdrSensor(BaseModel):
    endpoint_group: Optional[str] = None
    product_names: List[str] = Field(default=[])
    connectivity: Optional[str] = None
    version: Optional[str] = None
    last_connected_date_time: Optional[str] = None
    status: Optional[str] = None
    advanced_risk_telemetry_status: Optional[str] = None
    component_update_policy: Optional[str] = None
    component_update_status: Optional[str] = None
    kernel_support_package_version: Optional[str] = None
    patterns: List[EdrSensorPattern] = Field(default=[])


class EndpointSecurityEndpoint(BaseConsumable):
    endpoint_name: Optional[str] = None
    agent_guid: Optional[str] = None
    type: Optional[str] = None
    display_name: Optional[str] = None
    os_name: Optional[str] = None
    os_platform: Optional[str] = None
    os_version: Optional[str] = None
    os_kernel_version: Optional[str] = None
    os_architecture: Optional[str] = None
    last_used_ip: Optional[str] = None
    service_gateway_or_proxy: Optional[str] = None
    cpu_architecture: Optional[str] = None
    last_logged_on_user: Optional[str] = None
    isolation_status: Optional[str] = None
    ip_addresses: List[str] = Field(default=[])
    serial_number: Optional[str] = None
    version_control_policy: Optional[str] = None
    agent_update_policy: Optional[str] = None
    agent_update_status: Optional[str] = None
    credit_allocated_licenses: List[str] = Field(default=[])
    security_policy: Optional[str] = None
    security_policy_overridden_status: Optional[str] = None
    epp_agent: Optional[EppAgent] = None
    edr_sensor: Optional[EdrSensor] = None


class EndpointOs(BaseModel):
    name: Optional[str] = None
    platform: Optional[str] = None
    architecture: Optional[str] = None
    version: Optional[str] = None
    kernel_version: Optional[str] = None
    linux_distribution: Optional[str] = None


class EndpointInterface(BaseModel):
    ip_addresses: List[str] = Field(default=[])
    mac_address: Optional[str] = None


class EndpointDetailPattern(BaseModel):
    id: Optional[str] = None
    name: Optional[str] = None
    version: Optional[str] = None
    updated_date_time: Optional[str] = None


class EndpointDetailEngine(BaseModel):
    id: Optional[str] = None
    name: Optional[str] = None
    version: Optional[str] = None
    updated_date_time: Optional[str] = None


class EppAgentSubFeature(BaseModel):
    name: Optional[str] = None
    status: Optional[str] = None


class EppAgentFeature(BaseModel):
    name: Optional[str] = None
    status: Optional[str] = None
    outdated_pattern_ids: List[str] = Field(default=[])
    not_compliant_sub_features: List[EppAgentSubFeature] = Field(default=[])


class EndpointDetailVmDetail(BaseModel):
    key: Optional[str] = None
    value: Optional[str] = None


class EndpointDetailEppAgent(BaseModel):
    endpoint_group: Optional[str] = None
    protection_manager: Optional[str] = None
    product_names: List[str] = Field(default=[])
    policy_name: Optional[str] = None
    status: Optional[str] = None
    last_connected_date_time: Optional[str] = None
    tags: List[str] = Field(default=[])
    version: Optional[str] = None
    last_scanned_date_time: Optional[str] = None
    component_version: Optional[str] = None
    component_update_policy: Optional[str] = None
    component_update_status: Optional[str] = None
    kernel_support_package_version: Optional[str] = None
    installed_component_ids: List[str] = Field(default=[])
    domain_hierarchy: Optional[str] = None
    virtual_machine_details: List[EndpointDetailVmDetail] = Field(default=[])
    dsa_component_update_status: Optional[str] = None
    dsa_agent_update_status: Optional[str] = None
    dsa_component_updated_date_time: Optional[str] = None
    gcp_network_tags: List[str] = Field(default=[])
    docker_engine_version: Optional[str] = None
    patterns: List[EndpointDetailPattern] = Field(default=[])
    engines: List[EndpointDetailEngine] = Field(default=[])
    features: List[EppAgentFeature] = Field(default=[])


class DeepfakeDetector(BaseModel):
    status: Optional[str] = None


class EdrSettings(BaseModel):
    monitoring_level: Optional[str] = None
    deepfake_detector: Optional[DeepfakeDetector] = None


class EndpointDetailEdrSensor(BaseModel):
    endpoint_group: Optional[str] = None
    product_names: List[str] = Field(default=[])
    connectivity: Optional[str] = None
    last_connected_date_time: Optional[str] = None
    version: Optional[str] = None
    status: Optional[str] = None
    edr_settings: Optional[EdrSettings] = None
    advanced_risk_telemetry_status: Optional[str] = None
    component_update_policy: Optional[str] = None
    component_update_status: Optional[str] = None
    kernel_support_package_version: Optional[str] = None
    patterns: List[EndpointDetailPattern] = Field(default=[])


class EndpointDetail(BaseModel):
    endpoint_name: Optional[str] = None
    agent_guid: Optional[str] = None
    display_name: Optional[str] = None
    description: Optional[str] = None
    type: Optional[str] = None
    os: Optional[EndpointOs] = None
    last_used_ip: Optional[str] = None
    service_gateway_or_proxy: Optional[str] = None
    cpu_architecture: Optional[str] = None
    last_logged_on_user: Optional[str] = None
    isolation_status: Optional[str] = None
    recommended_actions: List[str] = Field(default=[])
    interfaces: List[EndpointInterface] = Field(default=[])
    serial_number: Optional[str] = None
    version_control_policy: Optional[str] = None
    agent_update_policy: Optional[str] = None
    agent_update_status: Optional[str] = None
    credit_allocated_licenses: List[str] = Field(default=[])
    security_policy: Optional[str] = None
    security_policy_overridden_status: Optional[str] = None
    epp_agent: Optional[EndpointDetailEppAgent] = None
    edr_sensor: Optional[EndpointDetailEdrSensor] = None


class EmailActivity(BaseConsumable):
    event_source_type: Optional[int] = None
    mail_msg_subject: Optional[str] = None
    mail_msg_id: Optional[str] = None
    msg_uuid: str
    mailbox: Optional[str] = None
    mail_sender_ip: Optional[str] = None
    mail_from_addresses: List[str] = Field(default=[])
    mail_whole_header: List[str] = Field(default=[])
    mail_to_addresses: List[str] = Field(default=[])
    mail_source_domain: Optional[str] = None
    search_d_l: Optional[str] = None
    scan_type: Optional[str] = None
    event_time: Optional[int] = None
    org_id: Optional[str] = None
    mail_urls_visible_link: List[str] = Field(default=[])
    mail_urls_real_link: List[str] = Field(default=[])


class EndpointActivity(BaseConsumable):
    dpt: Optional[int] = None
    dst: Optional[str] = None
    endpoint_guid: str
    endpoint_host_name: Optional[str] = None
    endpoint_ip: List[str] = Field(default=[])
    event_id: Optional[str] = None
    event_source_type: Optional[int] = None
    event_sub_id: Optional[str] = None
    object_integrity_level: Optional[IntegrityLevel] = None
    object_true_type: Optional[int] = None
    object_sub_true_type: Optional[int] = None
    win_event_id: Optional[int] = None
    event_time: Optional[int] = None
    event_time_d_t: Optional[str] = None
    host_name: Optional[str] = None
    logon_user: List[str] = Field(default=[])
    object_cmd: Optional[str] = None
    object_file_hash_sha1: Optional[str] = None
    object_file_path: Optional[str] = None
    object_host_name: Optional[str] = None
    object_ip: Optional[str] = None
    object_ips: List[str] = Field(default=[])
    object_port: Optional[int] = None
    object_registry_data: Optional[str] = None
    object_registry_key_handle: Optional[str] = None
    object_registry_value: Optional[str] = None
    object_signer: List[str] = Field(default=[])
    object_signer_valid: List[bool] = Field(default=[])
    object_user: Optional[str] = None
    os: Optional[str] = None
    parent_cmd: Optional[str] = None
    parent_file_hash_sha1: Optional[str] = None
    parent_file_path: Optional[str] = None
    process_cmd: Optional[str] = None
    process_file_hash_sha1: Optional[str] = None
    process_file_path: Optional[str] = None
    request: Optional[str] = None
    search_d_l: Optional[str] = None
    spt: Optional[int] = None
    src: Optional[str] = None
    src_file_hash_sha1: Optional[str] = None
    src_file_path: Optional[str] = None
    tags: List[str] = Field(default=[])
    uuid: Optional[str] = None

    @field_validator("event_sub_id", mode="before")
    @classmethod
    def map_event_sub_id(
        cls, value: Optional[Union[int, str]]
    ) -> Optional[str]:
        if value is not None and isinstance(value, int):
            return str(value)
        return value

    @field_validator("object_integrity_level", mode="before")
    @classmethod
    def map_object_integrity_level(
        cls, value: Optional[int]
    ) -> Optional[IntegrityLevel]:
        if value:
            if IntegrityLevel.UNTRUSTED <= value < IntegrityLevel.LOW:
                return IntegrityLevel.UNTRUSTED
            if IntegrityLevel.LOW <= value < IntegrityLevel.MEDIUM:
                return IntegrityLevel.LOW
            if IntegrityLevel.MEDIUM <= value < IntegrityLevel.HIGH:
                return IntegrityLevel.MEDIUM
            if IntegrityLevel.HIGH <= value < IntegrityLevel.SYSTEM:
                return IntegrityLevel.HIGH
            return IntegrityLevel.SYSTEM
        return None


class HostInfo(BaseModel):
    name: str
    ips: List[str]
    guid: str


class Entity(BaseModel):
    entity_id: str
    entity_type: EntityType
    entity_value: Union[str, HostInfo]
    related_entities: Union[List[str], List[List[str]]]
    related_indicator_ids: List[int]
    provenance: List[str]
    management_scope_group_id: Optional[str] = None
    management_scope_instance_id: Optional[str] = None
    management_scope_partition_key: Optional[str] = None


class Error(BaseModel):
    status: int
    code: Optional[str] = None
    message: Optional[str] = None
    number: Optional[int] = None


class ExceptionObject(BaseConsumable):
    value: str
    type: ObjectType
    last_modified_date_time: str
    description: Optional[str] = None

    @model_validator(mode="before")
    @classmethod
    def _map_data(cls, data: Dict[str, str]) -> Dict[str, str]:
        value = data.get(data.get("type", ""))
        if value:
            data["value"] = value
        return data


class ImpactScope(BaseModel):
    desktop_count: int
    server_count: int
    account_count: int
    email_address_count: int
    container_count: int
    cloud_identity_count: int
    entities: List[Entity]


class Indicator(BaseModel):
    id: int
    type: str
    value: Union[str, HostInfo]
    related_entities: Union[List[str], List[List[str]]]
    provenance: List[str]


class MatchedEvent(BaseModel):
    uuid: str
    matched_date_time: str
    type: str


class MatchedFilter(BaseModel):
    id: str
    name: str
    matched_date_time: str
    mitre_technique_ids: List[str]
    matched_events: List[MatchedEvent]


class MatchedIndicatorPattern(BaseModel):
    id: str
    pattern: str
    tags: List[str]
    matched_logs: List[str] = Field(default=[])


class MatchedRule(BaseModel):
    id: str
    name: str
    matched_filters: List[MatchedFilter]


class MsData(BaseModel):
    status: int
    task_id: Optional[str] = None

    @model_validator(mode="before")
    @classmethod
    def map_task_id(cls, data: Dict[str, Any]) -> Dict[str, Any]:
        data["task_id"] = _get_task_id(data)
        return data


class MsDataApiKey(MsData):
    id: str
    value: str
    expired_date_time: str

    @model_validator(mode="before")
    @classmethod
    def map_data(cls, data: Dict[str, Any]) -> Dict[str, Any]:
        data.update(data.pop("body", {}))
        return data


class MsDataUrl(MsData):
    url: str
    id: Optional[str] = None
    digest: Optional[Digest] = None

    @model_validator(mode="before")
    @classmethod
    def map_data(cls, data: Dict[str, Any]) -> Dict[str, Any]:
        data.update(data.pop("body", {}))
        return data


class MsError(Error):
    extra: Dict[str, str] = {}
    task_id: Optional[str] = None

    @model_validator(mode="before")
    @classmethod
    def map_data(cls, data: Dict[str, Any]) -> Dict[str, Any]:
        data.update(data.pop("body", {}))
        data.update(data.pop("error", {}))
        data["task_id"] = _get_task_id(data)
        url = data.pop("url", None)
        if url:
            data["extra"] = {"url": url}
        return data


class MsStatus(RootModel):
    root: List[int]

    def values(self) -> List[int]:
        return self.root


class OatEndpoint(BaseModel):
    endpoint_name: Optional[str] = None
    name: Optional[str] = None
    agent_guid: Optional[str] = None
    guid: Optional[str] = None
    ips: List[str]


class OatObject(BaseModel):
    field: str
    type: str
    value: Union[int, str, List[str]]
    master: Optional[bool] = None
    risk_level: Optional[OatRiskLevel] = None


class OatFilter(BaseModel):
    id: str
    unique_id: Optional[str] = None
    name: str
    description: Optional[str] = None
    mitre_tactic_ids: Optional[List[str]] = None
    tactics: Optional[List[str]] = None
    mitre_technique_ids: Optional[List[str]] = None
    techniques: Optional[List[str]] = None
    highlighted_objects: List[OatObject]
    risk_level: Optional[OatRiskLevel] = None
    level: Optional[OatRiskLevel] = None
    type: DetectionType


class OatEvent(BaseConsumable):
    source: Optional[OatDataSource] = None
    uuid: Optional[str] = None
    filters: List[OatFilter]
    endpoint: Optional[OatEndpoint] = None
    entity_type: OatEntityType
    entity_name: str
    detected_date_time: Optional[str] = None
    detection_time: Optional[str] = None
    ingested_date_time: Optional[str] = None
    detail: Union[EndpointActivity, EmailActivity]


class OatPackage(BaseConsumable):
    id: str
    created_date_time: str


class OatPipeline(BaseModel):
    has_detail: bool
    registered_date_time: str
    risk_levels: List[OatRiskLevel]
    id: Optional[str] = None
    description: Optional[str] = None


class SaeIndicator(Indicator):
    field: str
    filter_ids: List[str]


class SaeAlert(Alert):
    description: str
    matched_rules: List[MatchedRule]
    indicators: List[SaeIndicator]


class SandboxSuspiciousObject(BaseModel):
    risk_level: RiskLevel
    analysis_completion_date_time: str
    expired_date_time: str
    root_sha1: str
    type: ObjectType
    value: str

    @model_validator(mode="before")
    @classmethod
    def map_data(cls, data: Dict[str, str]) -> Dict[str, str]:
        obj = get_object(data)
        if obj:
            data["type"] = obj[0]
            data["value"] = obj[1]
        return data


class Script(BaseConsumable):
    id: str
    file_name: str
    file_type: ScriptType
    description: Optional[str] = None


class SuspiciousObject(ExceptionObject):
    scan_action: ScanAction
    risk_level: RiskLevel
    in_exception_list: bool
    expired_date_time: str


class TaskError(BaseModel):
    code: str
    message: str
    number: int


class TiIndicator(Indicator):
    fields: List[List[str]]
    matched_indicator_pattern_ids: List[str]
    first_seen_date_times: List[str]
    last_seen_date_times: List[str]


class TiAlert(Alert):
    campaign: Optional[str] = None
    industry: Optional[str] = None
    region_and_country: Optional[str] = None
    created_by: str
    total_indicator_count: int
    matched_indicator_count: int
    report_link: str
    matched_indicator_patterns: List[MatchedIndicatorPattern]
    indicators: List[TiIndicator]


def _get_task_id(data: Dict[str, Any]) -> Optional[str]:
    return next(
        map(
            lambda header: header.get("value", "").split("/")[-1],
            filter(
                lambda header: "Operation-Location" == header.get("name", ""),
                data.pop("headers", []),
            ),
        ),
        None,
    )


def get_object(data: Dict[str, str]) -> Optional[Tuple[str, str]]:
    obj = next(
        filter(
            lambda item: item[0] in map(lambda ot: ot.value, ObjectType),
            data.items(),
        ),
        None,
    )
    return obj



import typing
from typing import Any

from requests.adapters import DEFAULT_POOLBLOCK
from requests.adapters import HTTPAdapter as AdapterUrllib
from urllib3.connectionpool import HTTPConnectionPool as HTTPUrllib
from urllib3.connectionpool import HTTPSConnectionPool as HTTPSUrllib
from urllib3.poolmanager import PoolManager as ManagerUrllib


class HTTPConnectionPool(HTTPUrllib):
    @typing.no_type_check
    def urlopen(self, method, url, **kwargs):
        return super().urlopen(
            method,
            url,
            pool_timeout=5,
            **kwargs,
        )


class HTTPSConnectionPool(HTTPSUrllib, HTTPConnectionPool): ...


class PoolManager(ManagerUrllib):
    def __init__(
        self,
        **connection_pool_kw: Any,
    ):
        super().__init__(**connection_pool_kw)
        self.pool_classes_by_scheme = {
            "http": HTTPConnectionPool,
            "https": HTTPSConnectionPool,
        }


class HTTPAdapter(AdapterUrllib):
    @typing.no_type_check
    def init_poolmanager(
        self, connections, maxsize, block=DEFAULT_POOLBLOCK, **pool_kwargs
    ) -> None:
        super().init_poolmanager(connections, maxsize, block, **pool_kwargs)
        self.poolmanager = PoolManager(
            num_pools=connections,
            maxsize=maxsize,
            block=block,
            **pool_kwargs,
        )



import base64
import re
from typing import Any, Dict, List, Optional, Pattern, Type

from .model.enum import QueryOp, SearchMode, TaskAction
from .model.request import ObjectRequest, SuspiciousObjectRequest
from .model.response import (
    AccountTaskResp,
    BaseTaskResp,
    BlockListTaskResp,
    CollectFileTaskResp,
    CustomScriptTaskResp,
    EmailMessageTaskResp,
    EndpointTaskResp,
    SandboxSubmitUrlTaskResp,
    TerminateProcessTaskResp,
)

MAC_ADDRESS_PATTERN: Pattern[str] = re.compile(
    "^([0-9A-Fa-f]{2}[:-]){5}([0-9A-Fa-f]{2})$"
)
GUID_PATTERN: Pattern[str] = re.compile("^(\\w+-+){1,5}\\w+$")

TASK_ACTION_MAP: Dict[TaskAction, Type[BaseTaskResp]] = {
    TaskAction.COLLECT_FILE: CollectFileTaskResp,
    TaskAction.ISOLATE_ENDPOINT: EndpointTaskResp,
    TaskAction.RESTORE_ENDPOINT: EndpointTaskResp,
    TaskAction.TERMINATE_PROCESS: TerminateProcessTaskResp,
    TaskAction.QUARANTINE_MESSAGE: EmailMessageTaskResp,
    TaskAction.DELETE_MESSAGE: EmailMessageTaskResp,
    TaskAction.RESTORE_MESSAGE: EmailMessageTaskResp,
    TaskAction.BLOCK_SUSPICIOUS: BlockListTaskResp,
    TaskAction.REMOVE_SUSPICIOUS: BlockListTaskResp,
    TaskAction.RESET_PASSWORD: AccountTaskResp,
    TaskAction.SUBMIT_SANDBOX: SandboxSubmitUrlTaskResp,
    TaskAction.ENABLE_ACCOUNT: AccountTaskResp,
    TaskAction.DISABLE_ACCOUNT: AccountTaskResp,
    TaskAction.FORCE_SIGN_OUT: AccountTaskResp,
    TaskAction.RUN_CUSTOM_SCRIPT: CustomScriptTaskResp,
}


def _build_query(
    op: QueryOp, header: str, fields: Dict[str, str]
) -> Dict[str, str]:
    return filter_none(
        {
            header: (
                (" " + op + " ").join(
                    [f"{k} eq '{v}'" for k, v in fields.items()]
                )
            )
        }
    )


def _build_activity_query(
    op: QueryOp, fields: Dict[str, str]
) -> Dict[str, str]:
    return filter_none(
        {
            "TMV1-Query": (
                (" " + op + " ").join(
                    [f'{k}:"{v}"' for k, v in fields.items()]
                )
            )
        }
    )


def _b64_encode(value: Optional[str]) -> Optional[str]:
    return base64.b64encode(value.encode()).decode() if value else None


def build_activity_request(
    start_time: Optional[str],
    end_time: Optional[str],
    select: Optional[List[str]],
    top: int,
    search_mode: SearchMode,
) -> Dict[str, str]:
    return filter_none(
        {
            "startDateTime": start_time,
            "endDateTime": end_time,
            "select": ",".join(select) if select else select,
            "top": top,
            "mode": search_mode,
        }
    )


def build_object_request(*tasks: ObjectRequest) -> List[Dict[str, str]]:
    return [
        filter_none(
            {
                task.object_type.value: task.object_value,
                "description": task.description,
            }
        )
        for task in tasks
    ]


def build_sandbox_file_request(
    document_password: Optional[str],
    archive_password: Optional[str],
    arguments: Optional[str],
) -> Dict[str, str]:
    return filter_none(
        {
            "documentPassword": _b64_encode(document_password),
            "archivePassword": _b64_encode(archive_password),
            "arguments": _b64_encode(arguments),
        }
    )


def build_suspicious_request(
    *tasks: SuspiciousObjectRequest,
) -> List[Dict[str, Any]]:
    return [
        filter_none(
            {
                task.object_type.value: task.object_value,
                "description": task.description,
                "riskLevel": (
                    task.risk_level.value if task.risk_level else None
                ),
                "scanAction": (
                    task.scan_action.value if task.scan_action else None
                ),
                "daysToExpiration": task.days_to_expiration,
            }
        )
        for task in tasks
    ]


def filter_none(dictionary: Dict[str, Optional[Any]]) -> Dict[str, Any]:
    return {k: v for k, v in dictionary.items() if v}


def tmv1_filter(op: QueryOp, fields: Dict[str, str]) -> Dict[str, str]:
    return _build_query(op, "TMV1-Filter", fields)


def tmv1_query(op: QueryOp, fields: Dict[str, str]) -> Dict[str, str]:
    return _build_query(op, "TMV1-Query", fields)


def tmv1_activity_query(op: QueryOp, fields: Dict[str, str]) -> Dict[str, str]:
    return _build_activity_query(op, fields)


def filter_query(op: QueryOp, fields: Dict[str, str]) -> Dict[str, str]:
    return _build_query(op, "filter", fields)


def task_action_resp_class(
    task_action: TaskAction,
) -> Type[BaseTaskResp]:
    return TASK_ACTION_MAP.get(task_action, BaseTaskResp)



from typing import Callable, List, Optional

from .. import utils
from ..core import Core
from ..model.common import OatEvent, OatPackage
from ..model.enum import Api, HttpMethod, OatRiskLevel, QueryOp
from ..model.response import (
    ConsumeLinkableResp,
    GetOatPackageResp,
    GetPipelineResp,
    ListOatPackagesResp,
    ListOatPipelinesResp,
    ListOatsResp,
    MultiResp,
    NoContentResp,
    OatPipelineResp,
)
from ..result import MultiResult, Result


class Oat:
    _core: Core

    def __init__(self, core: Core):
        self._core = core

    def list(
        self,
        detected_start_date_time: Optional[str] = None,
        detected_end_date_time: Optional[str] = None,
        ingested_start_date_time: Optional[str] = None,
        ingested_end_date_time: Optional[str] = None,
        top: int = 50,
        op: QueryOp = QueryOp.AND,
        **fields: str,
    ) -> Result[ListOatsResp]:
        """Retrieves Observed Attack Techniques events in a paginated list.

        :param detected_start_date_time: Date that indicates the start of
        the event detection data retrieval time range (yyyy-MM-ddThh:mm:ssZ).
        Defaults to 24 hours before the request is made.
        :type detected_start_date_time: Optional[str]
        :param detected_end_date_time: Date that indicates the end of
        the event detection data retrieval time range (yyyy-MM-ddThh:mm:ssZ).
        Defaults to the time the request is made.
        :type detected_end_date_time: Optional[str]
        :param ingested_start_date_time: Date that indicates the start of
        the data ingestion time range (yyyy-MM-ddThh:mm:ssZ).
        :type ingested_start_date_time: Optional[str]
        :param ingested_end_date_time: Date that indicates the end of
        the data ingestion time range (yyyy-MM-ddThh:mm:ssZ).
        :type ingested_end_date_time: Optional[str]
        :param top: Number of records displayed on a page.
        :type top: int
        :param op: Operator to apply between fields (ie: ... OR ...).
        :type op: QueryOp
        :param fields: Field/value used to filter result (i.e:uuid="123"),
        check Vision One API documentation for full list of supported fields.
        :type fields: Dict[str, str]
        :rtype: Result[ListOatsResp]
        """
        return self._core.send(
            ListOatsResp,
            Api.GET_OAT_LIST,
            params=utils.filter_none(
                {
                    "detectedStartDateTime": detected_start_date_time,
                    "detectedEndDateTime": detected_end_date_time,
                    "ingestedStartDateTime": ingested_start_date_time,
                    "ingestedEndDateTime": ingested_end_date_time,
                    "top": top,
                }
            ),
            headers=utils.tmv1_filter(op, fields),
        )

    def consume(
        self,
        consumer: Callable[[OatEvent], None],
        detected_start_date_time: Optional[str] = None,
        detected_end_date_time: Optional[str] = None,
        ingested_start_date_time: Optional[str] = None,
        ingested_end_date_time: Optional[str] = None,
        top: int = 50,
        op: QueryOp = QueryOp.AND,
        **fields: str,
    ) -> Result[ConsumeLinkableResp]:
        """Retrieves and consume OAT events.

        :param consumer: Function which will consume every record in result.
        :type consumer: Callable[[Oat], None]
        :param detected_start_date_time: Date that indicates the start of
        the event detection data retrieval time range (yyyy-MM-ddThh:mm:ssZ).
        Defaults to 24 hours before the request is made.
        :type detected_start_date_time: Optional[str]
        :param detected_end_date_time: Date that indicates the end of
        the event detection data retrieval time range (yyyy-MM-ddThh:mm:ssZ).
        Defaults to the time the request is made.
        :type detected_end_date_time: Optional[str]
        :param ingested_start_date_time: Date that indicates the start of
        the data ingestion time range (yyyy-MM-ddThh:mm:ssZ).
        :type ingested_start_date_time: Optional[str]
        :param ingested_end_date_time: Date that indicates the end of
        the data ingestion time range (yyyy-MM-ddThh:mm:ssZ).
        :type ingested_end_date_time: Optional[str]
        :param top: Number of records displayed on a page.
        :type top: int
        :param op: Operator to apply between fields (ie: ... OR ...).
        :type op: QueryOp
        :param fields: Field/value used to filter result (i.e:uuid="123"),
        check Vision One API documentation for full list of supported fields.
        :type fields: Dict[str, str]
        :rtype: Result[ConsumeLinkableResp]
        """
        return self._core.send_linkable(
            ListOatsResp,
            Api.GET_OAT_LIST,
            consumer,
            params=utils.filter_none(
                {
                    "detectedStartDateTime": detected_start_date_time,
                    "detectedEndDateTime": detected_end_date_time,
                    "ingestedStartDateTime": ingested_start_date_time,
                    "ingestedEndDateTime": ingested_end_date_time,
                    "top": top,
                }
            ),
            headers=utils.tmv1_filter(op, fields),
        )

    def create_pipeline(
        self,
        has_detail: bool,
        risk_levels: List[OatRiskLevel],
        description: Optional[str] = None,
    ) -> Result[OatPipelineResp]:
        """Registers a customer
        to the Observed Attack Techniques data pipeline.

        :param has_detail: Retrieve detailed logs from the OAT data pipeline.
        :type has_detail: bool
        :param risk_levels: Risk levels to include in the results,
        requests must include at least one risk level.
        :type risk_levels: List[OatRiskLevel]
        :param description: Notes or comments about the pipeline.
        :type description: Optional[str]
        :return: Result[PipelineResp]
        """
        return self._core.send(
            OatPipelineResp,
            Api.CREATE_OAT_PIPELINE,
            HttpMethod.POST,
            json=utils.filter_none(
                {
                    "hasDetail": has_detail,
                    "riskLevels": risk_levels,
                    "description": description,
                }
            ),
        )

    def get_pipeline(self, pipeline_id: str) -> Result[GetPipelineResp]:
        """Displays the settings of the specified data pipeline.

        :param pipeline_id: Pipeline ID.
        :type pipeline_id: str
        :return: Result[GetPipelineResp]
        """
        return self._core.send(
            GetPipelineResp, Api.GET_OAT_PIPELINE.value.format(pipeline_id)
        )

    def update_pipeline(
        self,
        pipeline_id: str,
        etag: str,
        has_detail: Optional[bool] = None,
        risk_levels: Optional[List[OatRiskLevel]] = None,
        description: Optional[str] = None,
    ) -> Result[NoContentResp]:
        """Modifies the settings of the specified data pipeline.

        :param pipeline_id: Pipeline ID.
        :type pipeline_id: str
        :param etag: ETag of the resource you want to update.
        :type etag: str
        :param has_detail: Retrieve detailed logs from the OAT data pipeline.
        :type has_detail: Optional[bool]
        :param risk_levels: Risk levels to include in the results,
        requests must include at least one risk level.
        :type risk_levels: Optional[List[OatRiskLevel]]
        :param description: Notes or comments about the pipeline.
        :type description: Optional[str]
        :return: Result[NoContentResp]
        """
        return self._core.send(
            NoContentResp,
            Api.UPDATE_OAT_PIPELINE.value.format(pipeline_id),
            HttpMethod.PATCH,
            json=utils.filter_none(
                {
                    "hasDetail": has_detail,
                    "riskLevels": risk_levels,
                    "description": description,
                }
            ),
            headers={"If-Match": etag[1:-1] if etag.startswith('"') else etag},
        )

    def delete_pipelines(self, *pipeline_ids: str) -> MultiResult[MultiResp]:
        """Unregisters a customer
        from the Observed Attack Techniques data pipeline(s).

        :param pipeline_ids: Pipeline IDs.
        :type pipeline_ids: List[str]
        :return: MultiResult[MultiResp]
        """
        return self._core.send_multi(
            MultiResp,
            Api.DELETE_OAT_PIPELINE,
            json=[{"id": pipeline_id} for pipeline_id in pipeline_ids],
        )

    def list_pipelines(self) -> Result[ListOatPipelinesResp]:
        """Displays all data pipelines that have registered users.

        :return: Result[ListOatPipelineResp]
        """
        return self._core.send(ListOatPipelinesResp, Api.LIST_OAT_PIPELINE)

    def get_package(
        self, pipeline_id: str, package_id: str
    ) -> Result[GetOatPackageResp]:
        """Retrieves the specified Observed Attack Techniques package.

        :param pipeline_id: Pipeline ID.
        :type pipeline_id: str
        :param package_id: Package ID.
        :type package_id: str
        :return: Result[GetOatPackageResp]
        """
        return self._core.send(
            GetOatPackageResp,
            Api.DOWNLOAD_OAT_PACKAGE.value.format(pipeline_id, package_id),
        )

    def list_packages(
        self,
        pipeline_id: str,
        start_date_time: Optional[str] = None,
        end_date_time: Optional[str] = None,
        top: int = 500,
    ) -> Result[ListOatPackagesResp]:
        """Displays all the available packages from a data pipeline
         in a paginated list.

        :param pipeline_id: Pipeline ID.
        :type pipeline_id: str
        :param start_date_time: Date that indicates the start of
        the data retrieval time range. (yyyy-MM-ddThh:mm:ssZ).
        :type start_date_time: Optional[str]
        :param end_date_time: Date that indicates the end of
        the data retrieval time range. (yyyy-MM-ddThh:mm:ssZ).
        :type end_date_time: Optional[str]
        :param top: Number of records displayed on a page.
        :type top: int
        :return: Result[ListOatPackagesResp]
        """
        return self._core.send(
            ListOatPackagesResp,
            Api.LIST_OAT_PACKAGE.value.format(pipeline_id),
            params=utils.filter_none(
                {
                    "startDateTime": start_date_time,
                    "endDateTime": end_date_time,
                    "top": top,
                }
            ),
        )

    def consume_packages(
        self,
        pipeline_id: str,
        consumer: Callable[[OatPackage], None],
        start_date_time: Optional[str] = None,
        end_date_time: Optional[str] = None,
        top: int = 500,
    ) -> Result[ConsumeLinkableResp]:
        """Displays and consume all the available packages from a data pipeline
         in a paginated list.

        :param pipeline_id: Pipeline ID.
        :type pipeline_id: str
        :param consumer: Function which will consume every record in result.
        :type consumer: Callable[[OatPackage], None]
        :param start_date_time: Date that indicates the start of
        the data retrieval time range. (yyyy-MM-ddThh:mm:ssZ).
        :type start_date_time: Optional[str]
        :param end_date_time: Date that indicates the end of
        the data retrieval time range. (yyyy-MM-ddThh:mm:ssZ).
        :type end_date_time: Optional[str]
        :param top: Number of records displayed on a page.
        :type top: int
        :return: Result[ConsumeLinkableResp]
        """
        return self._core.send_linkable(
            ListOatPackagesResp,
            Api.LIST_OAT_PACKAGE.value.format(pipeline_id),
            consumer,
            params=utils.filter_none(
                {
                    "startDateTime": start_date_time,
                    "endDateTime": end_date_time,
                    "top": top,
                }
            ),
        )



from __future__ import annotations

import logging
import threading
from functools import lru_cache
from logging import Logger
from typing import Any

from . import api
from .core import Core

log: Logger = logging.getLogger(__name__)
lock = threading.Lock()


def init(
    name: str,
    token: str,
    url: str,
    pool_connections: int = 1,
    pool_maxsize: int = 1,
    connect_timeout: int = 10,
    read_timeout: int = 30,
) -> Client:
    """Synchronized Helper function to initialize a :class:`Client`.

    :param name: Identify the application using this library.
    :type name: str
    :param token: Authentication token created for your account.
    :type token: str
    :param url: Vision One API url this client connects to.
    :type url: str
    :param pool_connections: (optional) Number of connection to cache.
    :type pool_connections: int
    :param pool_maxsize: (optional) Maximum size of the pool.
    :type pool_maxsize: int
    :param connect_timeout: (optional) Seconds before connection timeout.
    :type connect_timeout: int
    :param read_timeout: (optional) Seconds before read timeout.
    :type connect_timeout: int
    :rtype: Client
    """
    lock.acquire()
    _cl = _client(
        appname=name,
        token=token,
        url=url,
        pool_connections=pool_connections,
        pool_maxsize=pool_maxsize,
        connect_timeout=connect_timeout,
        read_timeout=read_timeout,
    )
    lock.release()
    return _cl


@lru_cache(maxsize=1)
def _client(**kwargs: Any) -> Client:
    log.debug(
        "Initializing new client with [Appname=%s, Token=*****, URL=%s]",
        kwargs["appname"],
        kwargs["url"],
    )
    return Client(Core(**kwargs))


class Client:
    def __init__(self, core: Core):
        self._core = core
        self.account = api.Account(self._core)
        self.alert = api.Alert(self._core)
        self.api_key = api.ApiKey(self._core)
        self.email = api.Email(self._core)
        self.endpoint = api.Endpoint(self._core)
        self.note = api.Note(self._core)
        self.oat = api.Oat(self._core)
        self.object = api.Object(self._core)
        self.sandbox = api.Sandbox(self._core)
        self.script = api.CustomScript(self._core)
        self.system = api.System(self._core)
        self.task = api.Task(self._core)



from typing import Optional

from .. import utils
from ..core import Core
from ..model.enum import Api, HttpMethod
from ..model.response import (
    BytesResp,
    ListSandboxSuspiciousResp,
    MultiUrlResp,
    SandboxAnalysisResultResp,
    SandboxSubmissionStatusResp,
    SubmitFileToSandboxResp,
)
from ..result import MultiResult, Result


class Sandbox:
    _core: Core

    def __init__(self, core: Core):
        self._core = core

    def submit_file(
        self,
        file: bytes,
        file_name: str,
        document_password: Optional[str] = None,
        archive_password: Optional[str] = None,
        arguments: Optional[str] = None,
    ) -> Result[SubmitFileToSandboxResp]:
        """Submits a file to the sandbox for analysis.

        :param file: Raw content in bytes.
        :type file: bytes
        :param file_name: Name of the file.
        :type file_name: str
        :param document_password: Password used to
         decrypt the submitted file sample.
        :type document_password: Optional[str]
        :param archive_password: Password encoded in Base64 used to decrypt
         the submitted archive.
        :type archive_password: Optional[str]
        :param arguments: Command line arguments to run the submitted file.
         Only available for Portable Executable (PE) files and script files.
        :type arguments: Optional[str]
        :rtype: Result[SubmitFileToSandboxResp]:
        """
        return self._core.send(
            SubmitFileToSandboxResp,
            Api.SUBMIT_FILE_TO_SANDBOX,
            HttpMethod.POST,
            data=utils.build_sandbox_file_request(
                document_password, archive_password, arguments
            ),
            files={"file": (file_name, file, "application/octet-stream")},
        )

    def submit_url(self, *urls: str) -> MultiResult[MultiUrlResp]:
        """Submits URLs to the sandbox for analysis.

        :param urls: URL(s) to be submitted.
        :type urls: Tuple[str, ...]
        :rtype: MultiResult[MultiUrlResp]
        """
        return self._core.send_multi(
            MultiUrlResp,
            Api.SUBMIT_URLS_TO_SANDBOX,
            json=[{"url": url} for url in urls],
        )

    def download_analysis_result(
        self,
        submit_id: str,
        poll: bool = True,
        poll_time_sec: float = 1800,
    ) -> Result[BytesResp]:
        """Downloads the analysis results of the specified object as PDF.

        :param submit_id: Sandbox submission id.
        :type submit_id: str
        :param poll: If we should wait until the task is finished before
        to return the result.
        :type poll: bool
        :param poll_time_sec: Maximum time to wait for the result to
         be available.
        :type poll_time_sec: float
        :rtype: Result[BytesResp]:
        """
        return self._core.send_sandbox_result(
            BytesResp,
            Api.DOWNLOAD_SANDBOX_ANALYSIS_RESULT,
            submit_id,
            poll,
            poll_time_sec,
        )

    def download_investigation_package(
        self,
        submit_id: str,
        poll: bool = True,
        poll_time_sec: float = 1800,
    ) -> Result[BytesResp]:
        """Downloads the Investigation Package of the specified object.

        :param submit_id: Sandbox submission id.
        :type submit_id: str
        :param poll: If we should wait until the task is finished before
        to return the result.
        :type poll: bool
        :param poll_time_sec: Maximum time to wait for the result to
         be available.
        :type poll_time_sec: float
        :rtype: Result[BytesResp]:
        """
        return self._core.send_sandbox_result(
            BytesResp,
            Api.DOWNLOAD_SANDBOX_INVESTIGATION_PACKAGE,
            submit_id,
            poll,
            poll_time_sec,
        )

    def get_analysis_result(
        self,
        submit_id: str,
        poll: bool = True,
        poll_time_sec: float = 1800,
    ) -> Result[SandboxAnalysisResultResp]:
        """Retrieves the analysis results of the specified object.

        :param submit_id: Sandbox submission id.
        :type submit_id: str
        :param poll: If we should wait until the task is finished before
         to return the result.
        :type poll: bool
        :param poll_time_sec: Maximum time to wait for the result
         to be available.
        :type poll_time_sec: float
        :rtype: Result[SandboxAnalysisResultResp]:
        """
        return self._core.send_sandbox_result(
            SandboxAnalysisResultResp,
            Api.GET_SANDBOX_ANALYSIS_RESULT,
            submit_id,
            poll,
            poll_time_sec,
        )

    def get_submission_status(
        self, submit_id: str
    ) -> Result[SandboxSubmissionStatusResp]:
        """Retrieves the submission status of the specified object.

        :param submit_id: Sandbox submission id.
        :type submit_id: str
        :rtype: Result[SandboxSubmissionStatusResp]:
        """
        return self._core.send(
            SandboxSubmissionStatusResp,
            Api.GET_SANDBOX_SUBMISSION_STATUS.value.format(submit_id),
        )

    def list_suspicious(
        self,
        submit_id: str,
        poll: bool = True,
        poll_time_sec: float = 1800,
    ) -> Result[ListSandboxSuspiciousResp]:
        """Retrieves the suspicious object list associated to the
        specified object.

        :param submit_id: Sandbox submission id.
        :type submit_id: str
        :param poll: If we should wait until the task is finished before
         to return the result.
        :type poll: bool
        :param poll_time_sec: Maximum time to wait for the result
         to be available.
        :type poll_time_sec: float
        :rtype: Result[SandboxSuspiciousListResp]:
        """
        return self._core.send_sandbox_result(
            ListSandboxSuspiciousResp,
            Api.GET_SANDBOX_SUSPICIOUS_LIST,
            submit_id,
            poll,
            poll_time_sec,
        )



from __future__ import annotations

from typing import Any, Dict, Generic, List, Optional, TypeVar, Union

from pydantic import Field, field_validator, model_validator

from .common import (
    Account,
    AlertNote,
    ApiKey,
    BaseConsumable,
    BaseModel,
    Digest,
    EmailActivity,
    EmailMessage,
    Endpoint,
    EndpointActivity,
    EndpointDetail,
    EndpointSecurityEndpoint,
    ExceptionObject,
    MsData,
    MsDataApiKey,
    MsDataUrl,
    OatEvent,
    OatPackage,
    OatPipeline,
    SaeAlert,
    SandboxSuspiciousObject,
    Script,
    SuspiciousObject,
    TaskError,
    TiAlert,
    get_object,
)
from .enum import (
    ObjectType,
    RiskLevel,
    SandboxAction,
    SandboxObjectType,
    Status,
)

C = TypeVar("C", bound=BaseConsumable)
M = TypeVar("M", bound=MsData)


class BaseResponse(BaseModel):
    def __init__(self, **data: Any):
        super().__init__(**data)


class BaseLinkableResp(BaseResponse, Generic[C]):
    next_link: Optional[str] = None
    items: List[C]

    @model_validator(mode="before")
    @classmethod
    def map_data(cls, data: Dict[str, Any]) -> Dict[str, Any]:
        if not data["items"]:
            data["items"] = []
        return data


class BaseMultiResponse(BaseResponse, Generic[M]):
    items: List[M]


class BaseStatusResponse(BaseResponse):
    id: str
    status: Status
    created_date_time: str
    last_action_date_time: str


class BaseTaskResp(BaseStatusResponse):
    action: str
    description: Optional[str] = None
    account: Optional[str] = None
    error: Optional[TaskError] = None


MR = TypeVar("MR", bound=BaseMultiResponse[Any])
R = TypeVar("R", bound=BaseResponse)
S = TypeVar("S", bound=BaseStatusResponse)
T = TypeVar("T", bound=BaseTaskResp)


class AccountTaskResp(BaseTaskResp):
    tasks: List[Account]


class AddAlertNoteResp(BaseResponse):
    note_id: str = Field(validation_alias="Location")

    @field_validator("note_id", mode="before")
    @classmethod
    def get_id(cls, value: str) -> str:
        return _get_id(value)


class AddCustomScriptResp(BaseResponse):
    script_id: str = Field(validation_alias="Location")

    @field_validator("script_id", mode="before")
    @classmethod
    def get_id(cls, value: str) -> str:
        return _get_id(value)


class BlockListTaskResp(BaseTaskResp):
    type: ObjectType
    value: str

    @model_validator(mode="before")
    @classmethod
    def map_data(cls, data: Dict[str, str]) -> Dict[str, str]:
        obj = get_object(data)
        if obj:
            data["type"] = obj[0]
            data["value"] = obj[1]
        return data


class BytesResp(BaseResponse):
    content: bytes


class CollectFileTaskResp(BaseTaskResp):
    agent_guid: str
    endpoint_name: str
    file_path: Optional[str] = None
    file_sha1: Optional[str] = None
    file_sha256: Optional[str] = None
    file_size: Optional[int] = None
    resource_location: Optional[str] = None
    expired_date_time: Optional[str] = None
    password: Optional[str] = None


class ConnectivityResp(BaseResponse):
    status: str


class ConsumeLinkableResp(BaseResponse, alias_generator=None):
    total_consumed: int


class EndpointTaskResp(BaseTaskResp):
    agent_guid: str
    endpoint_name: str


class GetAlertResp(BaseResponse):
    data: Union[SaeAlert, TiAlert]
    etag: str


class GetAlertNoteResp(BaseResponse):
    data: AlertNote
    etag: str


class GetApiKeyResp(BaseResponse):
    data: ApiKey
    etag: str


class GetEndpointDetailsResp(BaseResponse):
    data: EndpointDetail


class GetOatPackageResp(BaseResponse):
    package: OatEvent


class GetPipelineResp(BaseResponse):
    data: OatPipeline
    etag: str


class ListAlertsResp(BaseLinkableResp[Union[SaeAlert, TiAlert]]):
    total_count: int
    count: int


class ListAlertNoteResp(BaseLinkableResp[AlertNote]): ...


class ListApiKeyResp(BaseLinkableResp[ApiKey]):
    total_count: int
    count: int


class ListCustomScriptsResp(BaseLinkableResp[Script]): ...


class ListEndpointActivityResp(BaseLinkableResp[EndpointActivity]):
    progress_rate: int


class GetEndpointActivitiesCountResp(BaseResponse):
    total_count: int


class ListEmailActivityResp(BaseLinkableResp[EmailActivity]):
    progress_rate: int


class GetEmailActivitiesCountResp(BaseResponse):
    total_count: int


class ListEndpointDataResp(BaseLinkableResp[Endpoint]): ...


class ListEndpointSecurityResp(BaseLinkableResp[EndpointSecurityEndpoint]):
    total_count: int
    count: int


class ListExceptionsResp(BaseLinkableResp[ExceptionObject]): ...


class ListOatsResp(BaseLinkableResp[OatEvent]):
    total_count: int
    count: int


class ListOatPipelinesResp(BaseResponse):
    items: List[OatPipeline]
    count: int


class ListOatPackagesResp(BaseLinkableResp[OatPackage]):
    total_count: int
    count: int
    requested_date_time: str
    latest_package_created_date_time: Optional[str] = None


class ListSuspiciousResp(BaseLinkableResp[SuspiciousObject]): ...


class MultiResp(BaseMultiResponse[MsData]): ...


class MultiUrlResp(BaseMultiResponse[MsDataUrl]): ...


class MultiApiKeyResp(BaseMultiResponse[MsDataApiKey]): ...


class NoContentResp(BaseResponse): ...


class EmailMessageTaskResp(BaseTaskResp):
    tasks: List[EmailMessage]


class CustomScriptTaskResp(BaseTaskResp):
    file_name: str
    agent_guid: str
    endpoint_name: str
    parameter: Optional[str] = None
    resource_location: Optional[str] = None
    expired_date_time: Optional[str] = None
    password: Optional[str] = None


class OatPipelineResp(BaseResponse):
    pipeline_id: str = Field(validation_alias="Location")

    @field_validator("pipeline_id", mode="before")
    @classmethod
    def get_id(cls, value: str) -> str:
        return _get_id(value)


class SubmitFileToSandboxResp(BaseResponse):
    id: str
    digest: Digest
    arguments: Optional[str] = None


class SandboxAnalysisResultResp(BaseResponse):
    id: str
    type: SandboxObjectType
    analysis_completion_date_time: str
    risk_level: RiskLevel
    true_file_type: Optional[str] = None
    digest: Optional[Digest] = None
    arguments: Optional[str] = None
    detection_names: List[str] = Field(default=[])
    threat_types: List[str] = Field(default=[])


class SandboxSubmissionStatusResp(BaseStatusResponse):
    action: SandboxAction
    resource_location: Optional[str] = None
    is_cached: Optional[bool] = None
    digest: Optional[Digest] = None
    arguments: Optional[str] = None


class ListSandboxSuspiciousResp(BaseResponse):
    items: List[SandboxSuspiciousObject]


class SandboxSubmitUrlTaskResp(BaseTaskResp):
    url: str
    sandbox_task_id: str


class TerminateProcessTaskResp(BaseTaskResp):
    agent_guid: str
    endpoint_name: str
    file_sha1: str
    file_name: Optional[str] = None


class TextResp(BaseResponse):
    text: str


def _get_id(location: str) -> str:
    return location.split("/")[-1]



from typing import Callable

from .. import utils
from ..core import Core
from ..model.common import ExceptionObject, SuspiciousObject
from ..model.enum import Api
from ..model.request import ObjectRequest, SuspiciousObjectRequest
from ..model.response import (
    ConsumeLinkableResp,
    ListExceptionsResp,
    ListSuspiciousResp,
    MultiResp,
)
from ..result import MultiResult, Result


class Object:
    _core: Core

    def __init__(self, core: Core):
        self._core = core

    def add_block(self, *objects: ObjectRequest) -> MultiResult[MultiResp]:
        """Adds object(s) to the Suspicious Object List,
        which blocks the objects on subsequent detections.

        :param objects: Object(s) to add.
        :type objects: Tuple[ObjectTask, ...]
        :rtype: MultiResult[MultiResp]
        """
        return self._core.send_multi(
            MultiResp,
            Api.ADD_TO_BLOCK_LIST,
            json=utils.build_object_request(*objects),
        )

    def delete_block(self, *objects: ObjectRequest) -> MultiResult[MultiResp]:
        """Removes object(s) that was added to the Suspicious Object List
          using the "Add to block list" action

        :param objects: Object(s) to remove.
        :type objects: Tuple[ObjectTask, ...]
        :rtype: MultiResult[MultiResp]
        """
        return self._core.send_multi(
            MultiResp,
            Api.REMOVE_FROM_BLOCK_LIST,
            json=utils.build_object_request(*objects),
        )

    def add_exception(self, *objects: ObjectRequest) -> MultiResult[MultiResp]:
        """Adds object(s) to the Exception List.

        :param objects: Object(s) to add.
        :type objects: Tuple[ObjectTask, ...]
        :rtype: MultiResult[MultiResp]
        """
        return self._core.send_multi(
            MultiResp,
            Api.ADD_TO_EXCEPTION_LIST,
            json=utils.build_object_request(*objects),
        )

    def delete_exception(
        self, *objects: ObjectRequest
    ) -> MultiResult[MultiResp]:
        """Removes object(s) from the Exception List.

        :param objects: Object(s) to remove.
        :type objects: Tuple[ObjectTask, ...]
        :rtype: MultiResult[MultiResp]
        """
        return self._core.send_multi(
            MultiResp,
            Api.REMOVE_FROM_EXCEPTION_LIST,
            json=utils.build_object_request(*objects),
        )

    def add_suspicious(
        self, *objects: SuspiciousObjectRequest
    ) -> MultiResult[MultiResp]:
        """Adds object(s) to the Suspicious Object List.

        :param objects: Object(s) to add.
        :type objects: Tuple[SuspiciousObjectTask, ...]
        :rtype: MultiResult[MultiResp]
        """
        return self._core.send_multi(
            MultiResp,
            Api.ADD_TO_SUSPICIOUS_LIST,
            json=utils.build_suspicious_request(*objects),
        )

    def delete_suspicious(
        self, *objects: ObjectRequest
    ) -> MultiResult[MultiResp]:
        """Removes object(s) from the Suspicious List.

        :param objects: Object(s) to remove.
        :type objects: Tuple[ObjectTask, ...]
        :rtype: MultiResult[MultiResp]
        """
        return self._core.send_multi(
            MultiResp,
            Api.REMOVE_FROM_SUSPICIOUS_LIST,
            json=utils.build_object_request(*objects),
        )

    def list_exception(self) -> Result[ListExceptionsResp]:
        """Retrieves exception objects in a paginated list.

        :rtype: Result[GetExceptionListResp]:
        """
        return self._core.send(ListExceptionsResp, Api.GET_EXCEPTION_OBJECTS)

    def list_suspicious(
        self,
    ) -> Result[ListSuspiciousResp]:
        """Retrieves suspicious objects in a paginated list.

        :rtype: Result[GetSuspiciousListResp]:
        """
        return self._core.send(ListSuspiciousResp, Api.GET_SUSPICIOUS_OBJECTS)

    def consume_exception(
        self, consumer: Callable[[ExceptionObject], None]
    ) -> Result[ConsumeLinkableResp]:
        """Retrieves and consume exception objects.

        :param consumer: Function which will consume every record in result.
        :type consumer: Callable[[ExceptionObject], None]
        :rtype: Result[ConsumeLinkableResp]:
        """
        return self._core.send_linkable(
            ListExceptionsResp, Api.GET_EXCEPTION_OBJECTS, consumer
        )

    def consume_suspicious(
        self, consumer: Callable[[SuspiciousObject], None]
    ) -> Result[ConsumeLinkableResp]:
        """Retrieves and consume suspicious objects.

        :param consumer: Function which will consume every record in result.
        :type consumer: Callable[[SuspiciousObject], None]
        :rtype: Result[ConsumeLinkableResp]:
        """
        return self._core.send_linkable(
            ListSuspiciousResp, Api.GET_SUSPICIOUS_OBJECTS, consumer
        )



from __future__ import annotations

from typing import Callable, Optional

from .. import utils
from ..core import Core
from ..model.common import ApiKey as Apk
from ..model.enum import Api, ApiStatus, HttpMethod, QueryOp
from ..model.request import ApiKeyRequest
from ..model.response import (
    ConsumeLinkableResp,
    GetApiKeyResp,
    ListApiKeyResp,
    MultiApiKeyResp,
    MultiResp,
    NoContentResp,
)
from ..result import MultiResult, Result


class ApiKey:
    _core: Core

    def __init__(self, core: Core):
        self._core = core

    def create(
        self,
        *keys: ApiKeyRequest,
    ) -> MultiResult[MultiApiKeyResp]:
        """Generates API keys designed to access Trend Vision One APIs.

        :param keys: API Key(s) to create.
        :type keys: Tuple[ApiKeyTask, ...]
        :return: MultiResult[MultiApiKeyResp]
        """
        return self._core.send_multi(
            MultiApiKeyResp,
            Api.CREATE_API_KEYS,
            json=[
                task.model_dump(by_alias=True, exclude_none=True)
                for task in keys
            ],
        )

    def get(self, key_id: str) -> Result[GetApiKeyResp]:
        """Retrieves the specified API key.

        :param key_id: Identifier of the API key.
        :type key_id: str
        :return: Result[GetApiKeyDetailsResp]
        """
        return self._core.send(
            GetApiKeyResp, Api.GET_API_KEY.value.format(key_id)
        )

    def update(
        self,
        key_id: str,
        etag: str,
        role: Optional[str] = None,
        name: Optional[str] = None,
        status: Optional[ApiStatus] = None,
        description: Optional[str] = None,
    ) -> Result[NoContentResp]:
        """Updates the specified API key.

        :param key_id: Identifier of the API key.
        :type key_id: str
        :param etag: ETag of the resource you want to update.
        :type etag: str
        :param role: User role assigned to the API key.
        :type role: Optional[str]
        :param name: Unique name of the API key.
        :type name: Optional[str]
        :param status: Status of an API key.
        :type status: Optional[ApiStatus]
        :param description: A brief note about the API key
        :type description: str
        :return: Result[NoContentResp]
        """
        return self._core.send(
            NoContentResp,
            Api.UPDATE_API_KEY.value.format(key_id),
            HttpMethod.PATCH,
            headers={
                "If-Match": etag if not etag.startswith('"') else etag[1:-1]
            },
            json=utils.filter_none(
                {
                    "role": role,
                    "name": name,
                    "status": status,
                    "description": description,
                }
            ),
        )

    def delete(self, *key_ids: str) -> MultiResult[MultiResp]:
        """Deletes the specified API keys.

        :param key_ids: Identifier of the API keys.
        :type key_ids: List[str]
        :return: MultiResult[MultiResp]
        """
        return self._core.send_multi(
            MultiResp,
            Api.DELETE_API_KEYS,
            json=[{"id": key_id} for key_id in key_ids],
        )

    def list(
        self, top: int = 50, op: QueryOp = QueryOp.AND, **fields: str
    ) -> Result[ListApiKeyResp]:
        """Retrieves API keys in a paginated list.

        :param top: Number of records displayed on a page.
        :type top: int
        :param op: Query operator to apply.
        :type op: QueryOp
        :param fields: Field/value used to filter result (ie: id="...")
        :type fields: Dict[str, str]
        check Vision One API documentation for full list of supported fields.
        :return: Result[GetApiKeyListResp]
        """
        return self._core.send(
            ListApiKeyResp,
            Api.GET_API_KEY_LIST,
            params={"orderBy": "createdDateTime desc", "top": top},
            headers=utils.tmv1_filter(op, fields),
        )

    def consume(
        self,
        consumer: Callable[[Apk], None],
        top: int = 50,
        op: QueryOp = QueryOp.AND,
        **fields: str,
    ) -> Result[ConsumeLinkableResp]:
        """Retrieves and consume API keys.

        :param consumer: Function which will consume every record in result.
        :type consumer: Callable[[ApiKey], None]
        :param top: Number of records displayed on a page.
        :type top: int
        :param op: Query operator to apply.
        :type op: QueryOp
        :param fields: Field/value used to filter result (ie: id="...")
        :type fields: Dict[str, str]
        check Vision One API documentation for full list of supported fields.
        :return: Result[GetApiKeyListResp]
        """
        return self._core.send_linkable(
            ListApiKeyResp,
            Api.GET_API_KEY_LIST,
            consumer,
            params={"orderBy": "createdDateTime desc", "top": top},
            headers=utils.tmv1_filter(op, fields),
        )



from ..core import Core
from ..model.enum import Api
from ..model.response import ConnectivityResp
from ..result import Result


class System:
    _core: Core

    def __init__(self, core: Core):
        self._core = core

    def check_connectivity(self) -> Result[ConnectivityResp]:
        """Checks the connection to the API service
        and verifies if your authentication token is valid.

        :rtype: Result[ConnectivityResp]
        """
        return self._core.send(ConnectivityResp, Api.CONNECTIVITY)



from .__about__ import __version__
from .client import Client, init
from .mapper import map_cef
from .model.common import (
    Account,
    Alert,
    AlertNote,
    ApiKey,
    DeepfakeDetector,
    Digest,
    EdrSensor,
    EdrSettings,
    EmailActivity,
    EmailMessage,
    Endpoint,
    EndpointActivity,
    EndpointDetail,
    EndpointDetailEdrSensor,
    EndpointDetailEngine,
    EndpointDetailEppAgent,
    EndpointDetailPattern,
    EndpointDetailVmDetail,
    EndpointInterface,
    EndpointOs,
    EndpointSecurityEndpoint,
    Entity,
    EppAgent,
    EppAgentFeature,
    EppAgentSubFeature,
    EppAgentVirtualMachineDetails,
    Error,
    ExceptionObject,
    HostInfo,
    ImpactScope,
    Indicator,
    MatchedEvent,
    MatchedFilter,
    MatchedIndicatorPattern,
    MatchedRule,
    MsData,
    MsDataApiKey,
    MsDataUrl,
    MsError,
    OatEndpoint,
    OatEvent,
    OatFilter,
    OatObject,
    OatPackage,
    OatPipeline,
    SaeAlert,
    SaeIndicator,
    SandboxSuspiciousObject,
    SuspiciousObject,
    TaskError,
    TiAlert,
    TiIndicator,
    Value,
    ValueList,
)
from .model.enum import (
    AlertStatus,
    ApiExpInMonths,
    ApiStatus,
    DetectionType,
    EntityType,
    Iam,
    IntegrityLevel,
    InvestigationResult,
    InvestigationStatus,
    OatDataSource,
    OatEntityType,
    OatRiskLevel,
    ObjectType,
    OperatingSystem,
    ProductCode,
    Provenance,
    Provider,
    QueryOp,
    RiskLevel,
    SandboxAction,
    SandboxObjectType,
    ScanAction,
    ScriptType,
    Severity,
    Status,
    TaskAction,
)
from .model.request import (
    AccountRequest,
    ApiKeyRequest,
    CollectFileRequest,
    CustomScriptRequest,
    EmailMessageIdRequest,
    EmailMessageUIdRequest,
    EndpointRequest,
    ObjectRequest,
    SuspiciousObjectRequest,
    TerminateProcessRequest,
)
from .model.response import (
    AccountTaskResp,
    AddAlertNoteResp,
    AddCustomScriptResp,
    BaseTaskResp,
    BlockListTaskResp,
    BytesResp,
    CollectFileTaskResp,
    ConnectivityResp,
    ConsumeLinkableResp,
    CustomScriptTaskResp,
    EmailMessageTaskResp,
    EndpointTaskResp,
    GetAlertNoteResp,
    GetAlertResp,
    GetApiKeyResp,
    GetEmailActivitiesCountResp,
    GetEndpointActivitiesCountResp,
    GetEndpointDetailsResp,
    GetOatPackageResp,
    GetPipelineResp,
    ListAlertNoteResp,
    ListAlertsResp,
    ListApiKeyResp,
    ListCustomScriptsResp,
    ListEmailActivityResp,
    ListEndpointActivityResp,
    ListEndpointDataResp,
    ListEndpointSecurityResp,
    ListExceptionsResp,
    ListOatPackagesResp,
    ListOatPipelinesResp,
    ListOatsResp,
    ListSandboxSuspiciousResp,
    ListSuspiciousResp,
    MultiApiKeyResp,
    MultiResp,
    MultiUrlResp,
    NoContentResp,
    OatPipelineResp,
    SandboxAnalysisResultResp,
    SandboxSubmissionStatusResp,
    SandboxSubmitUrlTaskResp,
    SubmitFileToSandboxResp,
    TerminateProcessTaskResp,
    TextResp,
)
from .result import MultiResult, Result, ResultCode

__all__ = [
    "__version__",
    "init",
    "map_cef",
    "Account",
    "AccountRequest",
    "AccountTaskResp",
    "AddAlertNoteResp",
    "AddCustomScriptResp",
    "Alert",
    "AlertNote",
    "AlertStatus",
    "ApiExpInMonths",
    "ApiKey",
    "ApiKeyRequest",
    "ApiStatus",
    "BaseTaskResp",
    "BlockListTaskResp",
    "BytesResp",
    "Client",
    "CollectFileRequest",
    "CollectFileTaskResp",
    "ConnectivityResp",
    "ConsumeLinkableResp",
    "CustomScriptRequest",
    "CustomScriptTaskResp",
    "DeepfakeDetector",
    "Digest",
    "EdrSettings",
    "EmailActivity",
    "EmailMessage",
    "EmailMessageIdRequest",
    "EmailMessageTaskResp",
    "EmailMessageUIdRequest",
    "Endpoint",
    "EndpointActivity",
    "EndpointDetail",
    "EndpointDetailEdrSensor",
    "EndpointDetailEngine",
    "EndpointDetailEppAgent",
    "EndpointDetailPattern",
    "EndpointDetailVmDetail",
    "EndpointInterface",
    "EndpointOs",
    "EndpointRequest",
    "EndpointSecurityEndpoint",
    "EndpointTaskResp",
    "EppAgent",
    "EppAgentFeature",
    "EppAgentSubFeature",
    "EppAgentVirtualMachineDetails",
    "Entity",
    "EntityType",
    "Error",
    "ExceptionObject",
    "ScriptType",
    "GetAlertResp",
    "GetAlertNoteResp",
    "GetApiKeyResp",
    "GetEmailActivitiesCountResp",
    "GetEndpointActivitiesCountResp",
    "GetEndpointDetailsResp",
    "GetOatPackageResp",
    "GetPipelineResp",
    "HostInfo",
    "Iam",
    "ImpactScope",
    "Indicator",
    "IntegrityLevel",
    "InvestigationResult",
    "InvestigationStatus",
    "ListAlertsResp",
    "ListAlertNoteResp",
    "ListApiKeyResp",
    "ListCustomScriptsResp",
    "ListEmailActivityResp",
    "ListEndpointActivityResp",
    "ListEndpointDataResp",
    "ListEndpointSecurityResp",
    "ListExceptionsResp",
    "ListOatsResp",
    "ListOatPackagesResp",
    "ListOatPipelinesResp",
    "ListSandboxSuspiciousResp",
    "ListSuspiciousResp",
    "MatchedEvent",
    "MatchedFilter",
    "MatchedIndicatorPattern",
    "MatchedRule",
    "MsData",
    "MsDataApiKey",
    "MsDataUrl",
    "MsError",
    "MultiApiKeyResp",
    "MultiResult",
    "MultiResp",
    "MultiUrlResp",
    "NoContentResp",
    "OatDataSource",
    "OatEndpoint",
    "OatEntityType",
    "OatEvent",
    "OatFilter",
    "OatObject",
    "OatPackage",
    "OatPipeline",
    "OatPipelineResp",
    "OatRiskLevel",
    "EdrSensor",
    "DetectionType",
    "ObjectRequest",
    "ObjectType",
    "OperatingSystem",
    "ProductCode",
    "Provenance",
    "Provider",
    "QueryOp",
    "Result",
    "ResultCode",
    "RiskLevel",
    "SaeAlert",
    "SaeIndicator",
    "SandboxAction",
    "SandboxAnalysisResultResp",
    "SandboxObjectType",
    "SandboxSubmissionStatusResp",
    "SandboxSubmitUrlTaskResp",
    "SandboxSuspiciousObject",
    "ScanAction",
    "Severity",
    "Status",
    "SubmitFileToSandboxResp",
    "SuspiciousObject",
    "SuspiciousObjectRequest",
    "TaskAction",
    "TaskError",
    "TerminateProcessRequest",
    "TerminateProcessTaskResp",
    "TextResp",
    "TiAlert",
    "TiIndicator",
    "Value",
    "ValueList",
]



from pytmv1.core import Core
from pytmv1.model.enum import Api

from ..model.request import AccountRequest
from ..model.response import MultiResp
from ..result import MultiResult


class Account:
    _core: Core

    def __init__(self, core: Core):
        self._core = core

    def disable(self, *accounts: AccountRequest) -> MultiResult[MultiResp]:
        """Signs the user out of all active application and browser sessions,
        and prevents the user from signing in any new session.

        :param accounts: Account(s) to disable.
        :type accounts: Tuple[AccountTask, ...]
        :rtype: MultiResult[MultiResp]
        """
        return self._core.send_multi(
            MultiResp,
            Api.DISABLE_ACCOUNT,
            json=[
                task.model_dump(by_alias=True, exclude_none=True)
                for task in accounts
            ],
        )

    def enable(self, *accounts: AccountRequest) -> MultiResult[MultiResp]:
        """Allows the user to sign in to new application and browser sessions.

        :param accounts: Account(s) to enable.
        :type accounts: Tuple[AccountTask, ...]
        :rtype: MultiResult[MultiResp]
        """
        return self._core.send_multi(
            MultiResp,
            Api.ENABLE_ACCOUNT,
            json=[
                task.model_dump(by_alias=True, exclude_none=True)
                for task in accounts
            ],
        )

    def reset(self, *accounts: AccountRequest) -> MultiResult[MultiResp]:
        """Signs the user out of all active application and browser sessions,
        and forces the user to create a new password during the next sign-in
        attempt.

        :param accounts: Account(s) to reset.
        :type accounts: Tuple[AccountTask, ...]
        :rtype: MultiResult[MultiResp]
        """
        return self._core.send_multi(
            MultiResp,
            Api.RESET_PASSWORD,
            json=[
                task.model_dump(by_alias=True, exclude_none=True)
                for task in accounts
            ],
        )

    def sign_out(self, *accounts: AccountRequest) -> MultiResult[MultiResp]:
        """Signs the user out of all active application and browser sessions.

        :param accounts: Account(s) to sign out.
        :type accounts: Tuple[AccountTask, ...]
        :rtype: MultiResult[MultiResp]
        """
        return self._core.send_multi(
            MultiResp,
            Api.SIGN_OUT_ACCOUNT,
            json=[
                task.model_dump(by_alias=True, exclude_none=True)
                for task in accounts
            ],
        )



from typing import Callable, Optional, Union

from .. import utils
from ..core import Core
from ..model.common import SaeAlert, TiAlert
from ..model.enum import (
    AlertStatus,
    Api,
    HttpMethod,
    InvestigationResult,
    InvestigationStatus,
    QueryOp,
)
from ..model.response import (
    ConsumeLinkableResp,
    GetAlertResp,
    ListAlertsResp,
    NoContentResp,
)
from ..result import Result


class Alert:
    _core: Core

    def __init__(self, core: Core):
        self._core = core

    def update_status(
        self,
        alert_id: str,
        etag: str,
        status: Optional[AlertStatus] = None,
        inv_result: Optional[InvestigationResult] = None,
        inv_status: Optional[InvestigationStatus] = None,
    ) -> Result[NoContentResp]:
        """Edit the status of an alert or investigation triggered in Workbench.

        :param alert_id: Workbench alert id.
        :type alert_id: str
        :param status: Status of a case or investigation.
        :type status: Optional[AlertStatus]
        :param inv_result: Findings of a case or investigation.
        :type inv_result: Optional[InvestigationResult]
        :param inv_status: (deprecated) Status of an investigation.
        :type inv_status: Optional[InvestigationStatus]
        :param etag: Target resource will be updated only if
         it matches ETag of the target one.
        :type etag: str
        :rtype: Result[NoContentResp]:
        """
        return self._core.send(
            NoContentResp,
            Api.UPDATE_ALERT_STATUS.value.format(alert_id),
            HttpMethod.PATCH,
            json=utils.filter_none(
                {
                    "status": status,
                    "investigationResult": inv_result,
                    "investigationStatus": inv_status,
                }
            ),
            headers={
                "If-Match": etag if etag.startswith('"') else '"' + etag + '"'
            },
        )

    def get(self, alert_id: str) -> Result[GetAlertResp]:
        """Displays information about the specified alert.

        :param alert_id: Workbench alert id.
        :type alert_id: str
        :rtype: Result[GetAlertDetailsResp]:
        """
        return self._core.send(
            GetAlertResp,
            Api.GET_ALERT.value.format(alert_id),
        )

    def list(
        self,
        start_time: Optional[str] = None,
        end_time: Optional[str] = None,
        op: QueryOp = QueryOp.AND,
        **fields: str,
    ) -> Result[ListAlertsResp]:
        """Retrieves workbench alerts in a paginated list.

        :param start_time: Date that indicates the start of the data retrieval
        time range (yyyy-MM-ddThh:mm:ssZ).
        Defaults to 24 hours before the request is made.
        :type start_time: Optional[str]
        :param end_time: Date that indicates the end of the data retrieval
        time range (yyyy-MM-ddThh:mm:ssZ).
        Defaults to the time the request is made.
        :type end_time: Optional[str]
        :param op: Operator to apply between fields (ie: ... OR ...).
        :type op: QueryOp
        :param fields: Field/value used to filter result (i.e:fileName="1.sh"),
        check Vision One API documentation for full list of supported fields.
        :type fields: Dict[str, str]
        :rtype: Result[GetAlertListResp]:
        """
        return self._core.send(
            ListAlertsResp,
            Api.GET_ALERT_LIST,
            params=utils.filter_none(
                {
                    "startDateTime": start_time,
                    "endDateTime": end_time,
                    "orderBy": "createdDateTime desc",
                }
            ),
            headers=utils.tmv1_filter(op, fields),
        )

    def consume(
        self,
        consumer: Callable[[Union[SaeAlert, TiAlert]], None],
        start_time: Optional[str] = None,
        end_time: Optional[str] = None,
        date_time_target: Optional[str] = "createdDateTime",
        op: QueryOp = QueryOp.AND,
        **fields: str,
    ) -> Result[ConsumeLinkableResp]:
        """Retrieves and consume workbench alerts.

        :param consumer: Function which will consume every record in result.
        :type consumer: Callable[[Union[SaeAlert, TiAlert]], None]
        :param start_time: Date that indicates the start of the data retrieval
        time range (yyyy-MM-ddThh:mm:ssZ).
        Defaults to 24 hours before the request is made.
        :type start_time: Optional[str]
        :param end_time: Date that indicates the end of the data retrieval
        time range (yyyy-MM-ddThh:mm:ssZ).
        Defaults to the time the request is made.
        :type end_time: Optional[str]
        :param op: Operator to apply between fields (ie: ... OR ...).
        :type op: QueryOp
        :param fields: Field/value used to filter result (i.e:fileName="1.sh"),
        check Vision One API documentation for full list of supported fields.
        :type fields: Dict[str, str]
        :rtype: Result[ConsumeLinkableResp]:
        """
        return self._core.send_linkable(
            ListAlertsResp,
            Api.GET_ALERT_LIST,
            consumer,
            params=utils.filter_none(
                {
                    "startDateTime": start_time,
                    "endDateTime": end_time,
                    "dateTimeTarget": date_time_target,
                    "orderBy": "createdDateTime desc",
                }
            ),
            headers=utils.tmv1_filter(op, fields),
        )



from .account import Account
from .alert import Alert
from .api_key import ApiKey
from .email import Email
from .endpoint import Endpoint
from .note import Note
from .oat import Oat
from .object import Object
from .sandbox import Sandbox
from .script import CustomScript
from .system import System
from .task import Task

__all__ = [
    "Account",
    "Alert",
    "ApiKey",
    "CustomScript",
    "Email",
    "Endpoint",
    "Note",
    "Oat",
    "Object",
    "Sandbox",
    "System",
    "Task",
]



from typing import Type

from ..core import Core
from ..model.response import BaseTaskResp, T
from ..result import Result


class Task:
    _core: Core

    def __init__(self, core: Core):
        self._core = core

    def get_result(
        self,
        task_id: str,
        poll: bool = True,
        poll_time_sec: float = 1800,
    ) -> Result[BaseTaskResp]:
        """Retrieves the result of a response task.

        :param task_id: Task id.
        :type task_id: str
        :param poll: If we should wait until the task is finished before
         to return the result.
        :type poll: bool
        :param poll_time_sec: Maximum time to wait for the result
        to be available.
        :type poll_time_sec: float
        :rtype: Result[T]:
        """
        return self.get_result_class(
            task_id, BaseTaskResp, poll, poll_time_sec
        )

    def get_result_class(
        self,
        task_id: str,
        class_: Type[T],
        poll: bool = True,
        poll_time_sec: float = 1800,
    ) -> Result[T]:
        """Retrieves the result of a response task.

        :param task_id: Task id.
        :type task_id: str
        :param class_: Expected task result class.
        :type class_: Type[T]
        :param poll: If we should wait until the task is finished before
         to return the result.
        :type poll: bool
        :param poll_time_sec: Maximum time to wait for the result
        to be available.
        :type poll_time_sec: float
        :rtype: Result[T]:
        """
        return self._core.send_task_result(
            class_, task_id, poll, poll_time_sec
        )



from typing import Callable, Optional

from .. import utils
from ..core import Core
from ..model.common import Script
from ..model.enum import Api, HttpMethod, QueryOp, ScriptType
from ..model.request import CustomScriptRequest
from ..model.response import (
    AddCustomScriptResp,
    ConsumeLinkableResp,
    ListCustomScriptsResp,
    MultiResp,
    NoContentResp,
    TextResp,
)
from ..result import MultiResult, Result


class CustomScript:
    _core: Core

    def __init__(self, core: Core):
        self._core = core

    def create(
        self,
        script_type: ScriptType,
        script_name: str,
        script_content: str,
        description: Optional[str] = None,
    ) -> Result[AddCustomScriptResp]:
        """
        Uploads a custom script. Supported file extensions: .ps1, .sh.
        Note: Custom scripts must use UTF-8 encoding.
        :param script_type: File type.
        :type script_type: ScriptType
        :param script_name: File name.
        :type script_name: str
        :param script_content: Plain text content of the script.
        :type script_content: str
        :param description: Description.
        :type description: Optional[str]
        :return: Result[AddACustomScriptResp]
        """
        return self._core.send(
            AddCustomScriptResp,
            Api.ADD_CUSTOM_SCRIPT,
            HttpMethod.POST,
            data=utils.filter_none(
                {"fileType": script_type.value, "description": description}
            ),
            files={
                "file": (
                    script_name,
                    bytes(script_content, "utf-8"),
                    "text/plain",
                )
            },
        )

    def update(
        self,
        script_id: str,
        script_type: ScriptType,
        script_name: str,
        script_content: str,
        description: Optional[str] = None,
    ) -> Result[NoContentResp]:
        """
        Updates a custom script. Supported file extensions: .ps1, .sh.
        Note: Custom scripts must use UTF-8 encoding.
        :param script_id: Unique string that identifies a script file.
        :type script_id: str
        :param script_type: File type.
        :type script_type: ScriptType
        :param script_name: File name.
        :type script_name: str
        :param script_content: Plain text content of the file.
        :type script_content: str
        :param description: Description.
        :type description: Optional[str]
        :return: Result[NoContentResp]
        """
        return self._core.send(
            NoContentResp,
            Api.UPDATE_CUSTOM_SCRIPT.value.format(script_id),
            HttpMethod.POST,
            data=utils.filter_none(
                {"fileType": script_type.value, "description": description}
            ),
            files={
                "file": (
                    script_name,
                    bytes(script_content, "utf-8"),
                    "text/plain",
                )
            },
        )

    def download(self, script_id: str) -> Result[TextResp]:
        """Downloads custom script.

        :param script_id: Unique string that identifies a script file.
        :type script_id: str
        :return: Result[BytesResp]
        """
        return self._core.send(
            TextResp, Api.DOWNLOAD_CUSTOM_SCRIPT.value.format(script_id)
        )

    def delete(self, script_id: str) -> Result[NoContentResp]:
        """Deletes custom script.

        :param script_id: Unique string that identifies a script file.
        :type script_id: str
        :return: Result[NoContentResp]
        """
        return self._core.send(
            NoContentResp,
            Api.DELETE_CUSTOM_SCRIPT.value.format(script_id),
            HttpMethod.DELETE,
        )

    def list(
        self, op: QueryOp = QueryOp.AND, **fields: str
    ) -> Result[ListCustomScriptsResp]:
        """Retrieves scripts in a paginated list filtered by provided values.

        :param op: Operator to apply between fields (ie: ... OR ...).
        :type op: QueryOp
        :param fields: Field/value used to filter result (i.e:fileName="1.sh"),
        check Vision One API documentation for full list of supported fields.
        :type fields: Dict[str, str]
        :return: Result[GetCustomScriptsResp]
        """
        return self._core.send(
            ListCustomScriptsResp,
            Api.GET_CUSTOM_SCRIPTS,
            params=utils.filter_query(op, fields),
        )

    def run(self, *scripts: CustomScriptRequest) -> MultiResult[MultiResp]:
        """Runs multiple custom script.

        :param scripts: Custom scripts to run.
        :type scripts: Tuple[CustomScriptTask, ...]
        :rtype: MultiResult[MultiResp]
        """
        return self._core.send_multi(
            MultiResp,
            Api.RUN_CUSTOM_SCRIPT,
            json=[
                task.model_dump(by_alias=True, exclude_none=True)
                for task in scripts
            ],
        )

    def consume(
        self,
        consumer: Callable[[Script], None],
        op: QueryOp = QueryOp.AND,
        **fields: str,
    ) -> Result[ConsumeLinkableResp]:
        """Retrieves and consume cust. scripts filtered by provided values.

        :param consumer: Function which will consume every record in result.
        :type consumer: Callable[[Script], None]
        :param op: Operator to apply between fields (ie: ... OR ...).
        :type op: QueryOp
        :param fields: Field/value used to filter result (i.e:fileName="1.sh"),
        check Vision One API documentation for full list of supported fields.
        :type fields: Dict[str, str]
        :return: Result[ConsumeLinkableResp]
        """
        return self._core.send_linkable(
            ListCustomScriptsResp,
            Api.GET_CUSTOM_SCRIPTS,
            consumer,
            params=utils.filter_query(op, fields),
        )



from enum import Enum


class AlertStatus(str, Enum):
    OPEN = "Open"
    IN_PROGRESS = "In Progress"
    CLOSED = "Closed"


class Api(str, Enum):
    CONNECTIVITY = "/healthcheck/connectivity"
    CREATE_API_KEYS = "/iam/apiKeys"
    GET_API_KEY_LIST = "/iam/apiKeys"
    GET_API_KEY = "/iam/apiKeys/{0}"
    UPDATE_API_KEY = "/iam/apiKeys/{0}"
    DELETE_API_KEYS = "/iam/apiKeys/delete"
    GET_ENDPOINT_DATA = "/eiqs/endpoints"
    GET_ENDPOINT_LIST = "/endpointSecurity/endpoints"
    GET_ENDPOINT_DETAILS = "/endpointSecurity/endpoints/{0}"
    GET_CUSTOM_SCRIPTS = "/response/customScripts"
    ADD_CUSTOM_SCRIPT = "/response/customScripts"
    DELETE_CUSTOM_SCRIPT = "/response/customScripts/{0}"
    DOWNLOAD_CUSTOM_SCRIPT = "/response/customScripts/{0}"
    UPDATE_CUSTOM_SCRIPT = "/response/customScripts/{0}/update"
    DELETE_EMAIL_MESSAGE = "/response/emails/delete"
    QUARANTINE_EMAIL_MESSAGE = "/response/emails/quarantine"
    RESTORE_EMAIL_MESSAGE = "/response/emails/restore"
    DISABLE_ACCOUNT = "/response/domainAccounts/disable"
    ENABLE_ACCOUNT = "/response/domainAccounts/enable"
    RESET_PASSWORD = "/response/domainAccounts/resetPassword"
    SIGN_OUT_ACCOUNT = "/response/domainAccounts/signOut"
    COLLECT_ENDPOINT_FILE = "/response/endpoints/collectFile"
    ISOLATE_ENDPOINT = "/response/endpoints/isolate"
    RESTORE_ENDPOINT = "/response/endpoints/restore"
    RUN_CUSTOM_SCRIPT = "/response/endpoints/runScript"
    TERMINATE_ENDPOINT_PROCESS = "/response/endpoints/terminateProcess"
    ADD_TO_BLOCK_LIST = "/response/suspiciousObjects"
    REMOVE_FROM_BLOCK_LIST = "/response/suspiciousObjects/delete"
    GET_TASK_RESULT = "/response/tasks/{0}"
    GET_SANDBOX_ANALYSIS_RESULT = "/sandbox/analysisResults/{0}"
    DOWNLOAD_SANDBOX_INVESTIGATION_PACKAGE = (
        "/sandbox/analysisResults/{0}/investigationPackage"
    )
    DOWNLOAD_SANDBOX_ANALYSIS_RESULT = "/sandbox/analysisResults/{0}/report"
    GET_SANDBOX_SUSPICIOUS_LIST = (
        "/sandbox/analysisResults/{0}/suspiciousObjects"
    )
    SUBMIT_FILE_TO_SANDBOX = "/sandbox/files/analyze"
    GET_SANDBOX_SUBMISSION_STATUS = "/sandbox/tasks/{0}"
    SUBMIT_URLS_TO_SANDBOX = "/sandbox/urls/analyze"
    GET_EMAIL_ACTIVITY_DATA = "/search/emailActivities"
    GET_ENDPOINT_ACTIVITY_DATA = "/search/endpointActivities"
    GET_SUSPICIOUS_OBJECTS = "/threatintel/suspiciousObjects"
    ADD_TO_SUSPICIOUS_LIST = "/threatintel/suspiciousObjects"
    REMOVE_FROM_SUSPICIOUS_LIST = "/threatintel/suspiciousObjects/delete"
    ADD_TO_EXCEPTION_LIST = "/threatintel/suspiciousObjectExceptions"
    GET_EXCEPTION_OBJECTS = "/threatintel/suspiciousObjectExceptions"
    REMOVE_FROM_EXCEPTION_LIST = (
        "/threatintel/suspiciousObjectExceptions/delete"
    )
    GET_ALERT_LIST = "/workbench/alerts"
    UPDATE_ALERT_STATUS = "/workbench/alerts/{0}"
    GET_ALERT = "/workbench/alerts/{0}"
    ADD_ALERT_NOTE = "/workbench/alerts/{0}/notes"
    GET_ALERT_NOTE_LIST = "/workbench/alerts/{0}/notes"
    GET_ALERT_NOTE = "/workbench/alerts/{0}/notes/{1}"
    UPDATE_ALERT_NOTE = "/workbench/alerts/{0}/notes/{1}"
    DELETE_ALERT_NOTE = "/workbench/alerts/{0}/notes/delete"
    GET_OAT_LIST = "/oat/detections"
    CREATE_OAT_PIPELINE = "/oat/dataPipelines"
    GET_OAT_PIPELINE = "/oat/dataPipelines/{0}"
    UPDATE_OAT_PIPELINE = "/oat/dataPipelines/{0}"
    DELETE_OAT_PIPELINE = "/oat/dataPipelines/delete"
    LIST_OAT_PIPELINE = "/oat/dataPipelines"
    DOWNLOAD_OAT_PACKAGE = "/oat/dataPipelines/{0}/packages/{1}"
    LIST_OAT_PACKAGE = "/oat/dataPipelines/{0}/packages"


class ApiExpInMonths(int, Enum):
    ONE = 1
    THREE = 3
    SIX = 6
    TWELVE = 12
    ZERO = 0


class ApiStatus(str, Enum):
    ENABLED = "enabled"
    DISABLED = "disabled"


class Iam(str, Enum):
    # Azure AD
    AAD = "AAD"
    # On-premise AD
    OPAD = "OPAD"


class IntegrityLevel(int, Enum):
    UNTRUSTED = 0
    LOW = 4096
    MEDIUM = 8192
    HIGH = 12288
    SYSTEM = 16384


class InvestigationResult(str, Enum):
    NO_FINDINGS = "No Findings"
    NOTEWORTHY = "Noteworthy"
    TRUE_POSITIVE = "True Positive"
    FALSE_POSITIVE = "False Positive"
    BENIGN_TRUE_POSITIVE = "Benign True Positive"


class InvestigationStatus(str, Enum):
    BENIGN_TRUE_POSITIVE = "Benign True Positive"
    CLOSED = "Closed"
    FALSE_POSITIVE = "False Positive"
    IN_PROGRESS = "In Progress"
    NEW = "New"
    TRUE_POSITIVE = "True Positive"


class EntityType(str, Enum):
    HOST = "host"
    ACCOUNT = "account"
    EMAIL_ADDRESS = "emailAddress"
    CONTAINER = "container"
    CLOUD_IDENTITY = "cloudIdentity"
    AWS_LAMBDA = "awsLambda"


class ScriptType(str, Enum):
    POWERSHELL = "powershell"
    BASH = "bash"


class HttpMethod(str, Enum):
    GET = "GET"
    PATCH = "PATCH"
    POST = "POST"
    PUT = "PUT"
    DELETE = "DELETE"


class OatDataSource(str, Enum):
    DETECTIONS = "detections"
    IDENTITY_ACTIVITY_DATA = "identityActivityData"
    ENDPOINT_ACTIVITY_DATA = "endpointActivityData"
    CLOUD_ACTIVITY_DATA = "cloudActivityData"
    EMAIL_ACTIVITY_DATA = "emailActivityData"
    MOBILE_ACTIVITY_DATA = "mobileActivityData"
    NETWORK_ACTIVITY_DATA = "networkActivityData"
    CONTAINER_ACTIVITY_DATA = "containerActivityData"


class OatEntityType(str, Enum):
    ENDPOINT = "endpoint"
    MAILBOX = "mailbox"
    CLOUDTRAIL = "cloudtrail"
    MESSAGING = "messaging"
    NETWORK = "network"
    ICS = "ics"
    CONTAINER = "container"


class OatRiskLevel(str, Enum):
    UNDEFINED = "undefined"
    INFO = "info"
    LOW = "low"
    MEDIUM = "medium"
    HIGH = "high"
    CRITICAL = "critical"


class DetectionType(str, Enum):
    CUSTOM = "custom"
    PRESET = "preset"


class ObjectType(str, Enum):
    IP = "ip"
    URL = "url"
    DOMAIN = "domain"
    FILE_SHA1 = "fileSha1"
    FILE_SHA256 = "fileSha256"
    SENDER_MAIL_ADDRESS = "senderMailAddress"


class OperatingSystem(str, Enum):
    LINUX = "Linux"
    WINDOWS = "Windows"
    MACOS = "macOS"
    MACOSX = "macOSX"


class ProductCode(str, Enum):
    SAO = "sao"
    SDS = "sds"
    XES = "xes"


class Provenance(str, Enum):
    ALERT = "Alert"
    SWEEPING = "Sweeping"
    NETWORK_ANALYTICS = "Network Analytics"


class Provider(str, Enum):
    SAE = "SAE"
    TI = "TI"


class QueryOp(str, Enum):
    AND = "and"
    OR = "or"


class RiskLevel(str, Enum):
    NO_RISK = "noRisk"
    LOW = "low"
    MEDIUM = "medium"
    HIGH = "high"


class SandboxAction(str, Enum):
    ANALYZE_FILE = "analyzeFile"
    ANALYZE_URL = "analyzeUrl"


class SandboxObjectType(str, Enum):
    URL = "url"
    FILE = "file"


class ScanAction(str, Enum):
    BLOCK = "block"
    LOG = "log"


class SearchMode(str, Enum):
    DEFAULT = "default"
    COUNT_ONLY = "countOnly"


class Severity(str, Enum):
    CRITICAL = "critical"
    HIGH = "high"
    MEDIUM = "medium"
    LOW = "low"


class Status(str, Enum):
    FAILED = "failed"
    QUEUED = "queued"
    REJECTED = "rejected"
    CANCELED = "canceled"
    RUNNING = "running"
    SUCCEEDED = "succeeded"
    PENDING_APPROVAL = "pendingApproval"


class TaskAction(str, Enum):
    COLLECT_FILE = "collectFile"
    COLLECT_EVIDENCE = "collectEvidence"
    COLLECT_NETWORK_ANALYSIS_PACKAGE = "collectNetworkAnalysisPackage"
    ISOLATE_ENDPOINT = "isolate"
    ISOLATE_ENDPOINT_MULTIPLE = "isolateForMultiple"
    RESTORE_ENDPOINT = "restoreIsolate"
    RESTORE_ENDPOINT_MULTIPLE = "restoreIsolateForMultiple"
    TERMINATE_PROCESS = "terminateProcess"
    DUMP_PROCESS_MEMORY = "dumpProcessMemory"
    QUARANTINE_MESSAGE = "quarantineMessage"
    DELETE_MESSAGE = "deleteMessage"
    RESTORE_MESSAGE = "restoreMessage"
    BLOCK_SUSPICIOUS = "block"
    REMOVE_SUSPICIOUS = "restoreBlock"
    RESET_PASSWORD = "resetPassword"
    SUBMIT_SANDBOX = "submitSandbox"
    ENABLE_ACCOUNT = "enableAccount"
    DISABLE_ACCOUNT = "disableAccount"
    FORCE_SIGN_OUT = "forceSignOut"
    REMOTE_SHELL = "remoteShell"
    RUN_INVESTIGATION_KIT = "runInvestigationKit"
    RUN_CUSTOM_SCRIPT = "runCustomScript"
    RUN_CUSTOM_SCRIPT_MULTIPLE = "runCustomScriptForMultiple"
    RUN_OS_QUERY = "runOsquery"
    RUN_YARA_RULES = "runYaraRules"



from typing import Callable, List, Optional

from .. import utils
from ..core import Core
from ..model.common import Endpoint as Ept
from ..model.common import EndpointActivity, EndpointSecurityEndpoint
from ..model.enum import Api, QueryOp, SearchMode
from ..model.request import (
    CollectFileRequest,
    EndpointRequest,
    TerminateProcessRequest,
)
from ..model.response import (
    ConsumeLinkableResp,
    GetEndpointActivitiesCountResp,
    GetEndpointDetailsResp,
    ListEndpointActivityResp,
    ListEndpointDataResp,
    ListEndpointSecurityResp,
    MultiResp,
)
from ..result import MultiResult, Result


class Endpoint:
    _core: Core

    def __init__(self, core: Core):
        self._core = core

    def isolate(self, *endpoints: EndpointRequest) -> MultiResult[MultiResp]:
        """Disconnects one or more endpoints from the network
        but allows communication with the managing Trend Micro server product.

        :param endpoints: Endpoint(s) to isolate.
        :type endpoints: Tuple[EndpointTask, ...]
        :rtype: MultiResult[MultiResp]
        """
        return self._core.send_endpoint(Api.ISOLATE_ENDPOINT, *endpoints)

    def restore(self, *endpoints: EndpointRequest) -> MultiResult[MultiResp]:
        """Restores network connectivity to one or more endpoints that applied
        the "Isolate endpoint" action.

        :param endpoints: Endpoint(s) to restore.
        :type endpoints: Tuple[EndpointTask, ...]
        :rtype: MultiResult[MultiResp]
        """
        return self._core.send_endpoint(Api.RESTORE_ENDPOINT, *endpoints)

    def collect_file(
        self, *files: CollectFileRequest
    ) -> MultiResult[MultiResp]:
        """Collects a file from one or more endpoints and then sends the files
        to Vision One in a password-protected archive.

        :param files: File(s) to collect.
        :type files: Tuple[FileTask, ...]
        :rtype: MultiResult[MultiResp]
        """
        return self._core.send_endpoint(Api.COLLECT_ENDPOINT_FILE, *files)

    def terminate_process(
        self, *processes: TerminateProcessRequest
    ) -> MultiResult[MultiResp]:
        """Terminates a process that is running on one or more endpoints.

        :param processes: Process(es) to terminate.
        :type processes: Tuple[ProcessTask, ...]
        :rtype: MultiResult[MultiResp]
        """
        return self._core.send_endpoint(
            Api.TERMINATE_ENDPOINT_PROCESS, *processes
        )

    def get_activity_count(
        self,
        start_time: Optional[str] = None,
        end_time: Optional[str] = None,
        select: Optional[List[str]] = None,
        top: int = 500,
        op: QueryOp = QueryOp.AND,
        **fields: str,
    ) -> Result[GetEndpointActivitiesCountResp]:
        """Retrieves the count of endpoint activity data in a paginated list
        filtered by provided values.

        :param start_time: Date that indicates the start of the data retrieval
        time range (yyyy-MM-ddThh:mm:ssZ).
        Defaults to 24 hours before the request is made.
        :type start_time: Optional[str]
        :param end_time: Date that indicates the end of the data retrieval
        time range (yyyy-MM-ddThh:mm:ssZ).
        Defaults to the time the request is made.
        :type end_time: Optional[str]
        :param select: List of fields to include in the search results,
        if no fields are specified, the query returns all supported fields.
        :type select: Optional[List[str]]
        :param top: Number of records fetched per page.
        :type top: int
        :param op: Operator to apply between fields (ie: dpt=... OR src=...)
        :type op: QueryOp
        :param fields: Field/value used to filter result (ie: dpt="443")
        check Vision One API documentation for full list of supported fields.
        :type fields: Dict[str, str]
        :rtype: Result[GetEndpointActivityDataCountResp]:
        """
        return self._core.send(
            GetEndpointActivitiesCountResp,
            Api.GET_ENDPOINT_ACTIVITY_DATA,
            params=utils.build_activity_request(
                start_time,
                end_time,
                select,
                top,
                SearchMode.COUNT_ONLY,
            ),
            headers=utils.tmv1_activity_query(op, fields),
        )

    def list_data(
        self, op: QueryOp = QueryOp.AND, **fields: str
    ) -> Result[ListEndpointDataResp]:
        """Retrieves endpoints in a paginated list filtered by provided values.

        :param op: Query operator to apply.
        :type op: QueryOp
        :param fields: Field/value used to filter result (i.e:ip="1.1.1.1")
        check Vision One API documentation for full list of supported fields.
        :type fields: Dict[str, str]
        :rtype: Result[GetEndpointDataResp]:
        """
        return self._core.send(
            ListEndpointDataResp,
            Api.GET_ENDPOINT_DATA,
            headers=utils.tmv1_query(op, fields),
        )

    def list_activity(
        self,
        start_time: Optional[str] = None,
        end_time: Optional[str] = None,
        select: Optional[List[str]] = None,
        top: int = 500,
        op: QueryOp = QueryOp.AND,
        **fields: str,
    ) -> Result[ListEndpointActivityResp]:
        """Retrieves endpoint activity data in a paginated list
         filtered by provided values.

        :param start_time: Date that indicates the start of the data retrieval
        time range (yyyy-MM-ddThh:mm:ssZ).
        Defaults to 24 hours before the request is made.
        :type start_time: Optional[str]
        :param end_time: Date that indicates the end of the data retrieval
        time range (yyyy-MM-ddThh:mm:ssZ).
        Defaults to the time the request is made.
        :type end_time: Optional[str]
        :param select: List of fields to include in the search results,
        if no fields are specified, the query returns all supported fields.
        :type select: Optional[List[str]]
        :param top: Number of records fetched per page.
        :type top: int
        :param op: Operator to apply between fields (ie: dpt=... OR src=...)
        :type op: QueryOp
        :param fields: Field/value used to filter result (ie: dpt="443")
        check Vision One API documentation for full list of supported fields.
        :type fields: Dict[str, str]
        :rtype: Result[GetEndpointActivityDataResp]:
        """
        return self._core.send(
            ListEndpointActivityResp,
            Api.GET_ENDPOINT_ACTIVITY_DATA,
            params=utils.build_activity_request(
                start_time,
                end_time,
                select,
                top,
                SearchMode.DEFAULT,
            ),
            headers=utils.tmv1_activity_query(op, fields),
        )

    def consume_data(
        self,
        consumer: Callable[[Ept], None],
        op: QueryOp,
        **fields: str,
    ) -> Result[ConsumeLinkableResp]:
        """Retrieves and consume endpoints.

        :param consumer: Function which will consume every record in result.
        :type consumer: Callable[[Endpoint], None]
        :param op: Query operator to apply.
        :type op: QueryOp
        :param fields: Field/value used to filter result (i.e:ip="1.1.1.1")
        check Vision One API documentation for full list of supported fields.
        :type fields: Dict[str, str]
        :rtype: Result[ConsumeLinkableResp]:
        """
        return self._core.send_linkable(
            ListEndpointDataResp,
            Api.GET_ENDPOINT_DATA,
            consumer,
            headers=utils.tmv1_query(op, fields),
        )

    def consume_activity(
        self,
        consumer: Callable[[EndpointActivity], None],
        start_time: Optional[str] = None,
        end_time: Optional[str] = None,
        select: Optional[List[str]] = None,
        top: int = 500,
        op: QueryOp = QueryOp.AND,
        **fields: str,
    ) -> Result[ConsumeLinkableResp]:
        """Retrieves and consume endpoint activity data in a paginated list
         filtered by provided values.

        :param consumer: Function which will consume every record in result.
        :type consumer: Callable[[EndpointActivity], None]
        :param start_time: Date that indicates the start of the data retrieval
        time range (yyyy-MM-ddThh:mm:ssZ).
        Defaults to 24 hours before the request is made.
        :type start_time: Optional[str]
        :param end_time: Date that indicates the end of the data retrieval
        time range (yyyy-MM-ddThh:mm:ssZ).
        Defaults to the time the request is made.
        :type end_time: Optional[str]
        :param select: List of fields to include in the search results,
        if no fields are specified, the query returns all supported fields.
        :type select: Optional[List[str]]
        :param top: Number of records fetched per page.
        :type top: int
        :param op: Operator to apply between fields (ie: dpt=... OR src=...)
        :type op: QueryOp
        :param fields: Field/value used to filter result (ie: dpt="443")
        check Vision One API documentation for full list of supported fields.
        :type fields: Dict[str, str]
        :rtype: Result[ConsumeLinkableResp]:
        """
        return self._core.send_linkable(
            ListEndpointActivityResp,
            Api.GET_ENDPOINT_ACTIVITY_DATA,
            consumer,
            params=utils.build_activity_request(
                start_time,
                end_time,
                select,
                top,
                SearchMode.DEFAULT,
            ),
            headers=utils.tmv1_activity_query(op, fields),
        )

    def get_endpoint(self, endpoint_id: str) -> Result[GetEndpointDetailsResp]:
        """Displays the detailed profile of the specified endpoint.

        :param endpoint_id: The ID of the endpoint
        on the Vision One platform.
        :type endpoint_id: str
        :rtype: Result[GetEndpointDetailsResp]
        """
        return self._core.send(
            GetEndpointDetailsResp,
            Api.GET_ENDPOINT_DETAILS.value.format(endpoint_id),
        )

    def list_endpoints(
        self,
        select: Optional[List[str]] = None,
        top: int = 100,
        order_by: Optional[str] = None,
        op: QueryOp = QueryOp.AND,
        **fields: str,
    ) -> Result[ListEndpointSecurityResp]:
        """Displays a detailed list of your endpoints.

        :param select: List of endpoint information fields to include
        in the response; if no fields are specified, all fields are returned.
        :type select: Optional[List[str]]
        :param top: Number of records displayed on a page.
        :type top: int
        :param order_by: Field by which the results are sorted
        (e.g. "agentGuid desc").
        :type order_by: Optional[str]
        :param op: Query operator to apply.
        :type op: QueryOp
        :param fields: Field/value used to filter result
        (i.e: osPlatform="windows"),
        check Vision One API documentation for full list of supported fields.
        :type fields: Dict[str, str]
        :rtype: Result[ListEndpointSecurityResp]
        """
        return self._core.send(
            ListEndpointSecurityResp,
            Api.GET_ENDPOINT_LIST,
            params=utils.filter_none(
                {
                    "select": ",".join(select) if select else None,
                    "top": top,
                    "orderBy": order_by,
                }
            ),
            headers=utils.tmv1_filter(op, fields),
        )

    def consume_endpoints(
        self,
        consumer: Callable[[EndpointSecurityEndpoint], None],
        select: Optional[List[str]] = None,
        top: int = 100,
        order_by: Optional[str] = None,
        op: QueryOp = QueryOp.AND,
        **fields: str,
    ) -> Result[ConsumeLinkableResp]:
        """Retrieves and consume endpoint security endpoints.

        :param consumer: Function which will consume every record in result.
        :type consumer: Callable[[EndpointSecurityEndpoint], None]
        :param select: List of endpoint information fields to include
        in the response; if no fields are specified, all fields are returned.
        :type select: Optional[List[str]]
        :param top: Number of records displayed on a page.
        :type top: int
        :param order_by: Field by which the results are sorted
        (e.g. "agentGuid desc").
        :type order_by: Optional[str]
        :param op: Query operator to apply.
        :type op: QueryOp
        :param fields: Field/value used to filter result
        (i.e: osPlatform="windows"),
        check Vision One API documentation for full list of supported fields.
        :type fields: Dict[str, str]
        :rtype: Result[ConsumeLinkableResp]
        """
        return self._core.send_linkable(
            ListEndpointSecurityResp,
            Api.GET_ENDPOINT_LIST,
            consumer,
            params=utils.filter_none(
                {
                    "select": ",".join(select) if select else None,
                    "top": top,
                    "orderBy": order_by,
                }
            ),
            headers=utils.tmv1_filter(op, fields),
        )
